# # # # # # # # # # # # # # # # # # # # # # #

# ADDR.S - Address Information Database and Dialer
# CIEX (C) 1993, 1994 New Line Software, Inc.
# Script version 1.2

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Execute a database manager for personal address information.
# The program also includes a Hayes compatible phone dialer.

# # # # # # # # # # # # # # # # # # # # # # #

# Initialize variables, and screen parameters on entry.

set -clear          # Remove all variables and callpoints.
screen -default     # Initialize default screen parameters.
box -style single   # Boxes are drawn as single lines.
source -calls       # Speed-up access to callpoints in this file.

# These are the labels for the 9 data fields.

set $label:1 = 'Last Name'
set $label:2 = 'First Name'
set $label:3 = 'Company'
set $label:4 = 'Phone Number'
set $label:5 = 'Street / Box'
set $label:6 = 'City'
set $label:7 = 'State'
set $label:8 = 'ZIP'
set $label:9 = 'Comments'

# These are the colors used by the program:

set $background_color = black
set $box_color        = red
set $highlight_color  = cyan
set $label_color      = green
set $text_color       = white

# Location of the database file is:

if \ciex\data is directory
   set $dbfile = "\ciex\data\addr.dat"
else
   set $dbfile = "addr.dat"
endif

# Other parameters:

set $this_record   = 1    # Default initial record.
set $record_count  = 0    # Default number of records.
set $max_records   = 50   # Maximum number of records.
set $field:width   = 55   # Width of input fields.
set $field:xorigin = 23   # X origin of input fields.

set $field:format  = ["-scr-" & $field:width ]

# Restore the database from the disk, if it exists.

if $dbfile is file
   setblock -file $static = $dbfile
   set $record_count = [ $static:dbase^range ]
endif

# If no database exists, initialize it as follows:

if $static:dbase^range is null
   call init_record
endif

# Communication parameters:

set $static:comm:port    ? 2   # Default comm port
set $static:comm:timeout ? 5   # Default time to auto-hangup.

# Get the current date using internal DATE command.

set -cmd $date = startblock
   echo ""|DATE
endblock

if $date^range is null    # Date command failed.

   set $date = 1994-1-1

else # Process the date as follows:

   set -list $date = $date:1
   set -list $date = $date:5 by '-'
   set $date = [ "$date:3"  "-" "$date:1" "-" "$date:2" ]

endif

callpoint: begin

# Draw the initial screen.

screen -buffer on      # Turn on screen buffering.
call draw_form         # Draw the Address entry form.
call update_form       # Update the form with the first record.
screen -flush          # Flush the screen buffer to the screen.
screen -buffer off     # Turn off screen buffering.

# Begin the main service loop.  Loop to get user input:

loop

   # Get a key. The program stops until a key is entered.

   read -key $key

   # The name of the key now resides in the variable '$key'
   # Decode the key as follows.

   if      -test $key eq f1    # Display the help screen

      call display_help_screen

   else if -test $key eq f2    # Add a record at current position.

      call add_record
      set $data_changed = true

   else if -test $key eq f3    # Edit the current record.

      call edit_record
      set $data_changed = true

   else if -test $key eq f4    # Delete the current record.

      call delete_record
      set $data_changed = true

   else if -test $key eq f5    # Display the "dialer" control screen.

      call dial_number

   else if -test $key eq f8    # Display the "sort" control screen.

      call sort_records
      set $data_changed = true

   else if -test $key eq f9    # Display the "find" control screen.

      call find_record

   else if -test $key eq f10   # End the program.

      screen -buffer off
      call end_session

   else if -test $key eq pgup  # Previous record.

      if $this_record gt 1
         set $this_record - 1
      endif

   else if -test $key eq pgdn  # Next record.

      if $this_record lt $record_count
         set $this_record + 1
      endif

   else if -test $key eq home  # First record

      set $this_record = 1

   else if -test $key eq end   # Last record.

      set $this_record = $static:dbase^last

   else

      break -next # No such key. Return to prompt.

   endif

   # The key has been processed.
   # Display the current record.

   screen -buffer on      # Turn on screen buffering.
   call draw_form         # Draw the Address entry form.
   call update_form       # Update the form with the first record.
   screen -flush          # Flush the screen buffer to the screen.
   screen -buffer off     # Turn off screen buffering.

endloop

return

# # # # # # # # # # # # # # # #

callpoint: draw_form

# Draw the database form.

# # # # # # # # # # # # # # # #

# Setup the screen colors.

screen -background $background_color
screen -clear
screen -fore $box_color

# Draw the fields of the form.

move 1  2;   line; move 1  4;   line
move 1  6;   line; move 1  8;   line
move 1 10;   line; move 1 12;   line
move 1 14;   line; move 1 16;   line
move 1 18;   line; move 1 20;   line
move 1 23;   line;

move 1  2;   line -vertical 19
move 20 2;   line -vertical 19

# Add the field labels:

screen -fore $label_color
move 4 3    ; echo -screen15 $label:1
move -down 2; echo -screen15 $label:2
move -down 2; echo -screen15 $label:3
move -down 2; echo -screen15 $label:4
move -down 2; echo -screen15 $label:5
move -down 2; echo -screen15 $label:6
move -down 2; echo -screen15 $label:7
move -down 2; echo -screen15 $label:8
move -down 2; echo -screen15 $label:9

# Outline of form is drawn. Return.

return

# # # # # # # # # # # # # # # #

callpoint: update_form

# Update the form with the current record information.

# # # # # # # # # # # # # # # #

# Display the top legend.

screen -fore $highlight_color
move 1 1
line -clear
echo -center ~
" F1=Help   F2=Add   F3=Edit   F4=Del   F5=Dial   F8=Sort   F9=Find   F10=Exit"
line -invert 80

# Display the bottom legend.

move -home
line -clear
echo -center ~
" PgUp / PgDn / Home / End                        CIEX V2.3, (c) NLS, Inc."
line -invert 80

# Display record identification:

screen -fore $label_color
move 6 21
echo -screen ~
'RECORD:      OF:       CREATED:                  MODIFIED:                 '

# Display the record data:

screen -fore $text_color
move $field:xorigin 1

move -down  2; echo $field:format $static:dbase:$this_record:1
move -down  2; echo $field:format $static:dbase:$this_record:2
move -down  2; echo $field:format $static:dbase:$this_record:3
move -down  2; echo $field:format $static:dbase:$this_record:4
move -down  2; echo $field:format $static:dbase:$this_record:5
move -down  2; echo $field:format $static:dbase:$this_record:6
move -down  2; echo $field:format $static:dbase:$this_record:7
move -down  2; echo $field:format $static:dbase:$this_record:8
move -down  2; echo $field:format $static:dbase:$this_record:9

# Update the record identification:

move 14 21    ; echo -scr3 $this_record
move -right 9 ; echo -scr3 $record_count

# Dates require special handling to convert to MM-DD-YYYY

set -list $disp_date = $static:dbase:$this_record:created by '-'
set $disp_date = ["$disp_date:2"  "-" "$disp_date:3" "-" "$disp_date:1"]

# Display the date this record was created.

move -right 16 ; echo -scr-10 $disp_date

set -list $disp_date = $static:dbase:$this_record:modified by '-'
set $disp_date = ["$disp_date:2"  "-" "$disp_date:3" "-" "$disp_date:1"]

# Display the date this record was last modified.

move -right 27 ; echo -scr-10 $disp_date

# The address record is now being displayed. Return

return

# # # # # # # # # # # # # # # #

callpoint: add_record

# Add a record following the current record.

# # # # # # # # # # # # # # # #

if $record_count eq 0

   call edit_record
   return

else if $record_count ge $max_records

   # Maximum number of records exists. Display an error.

   move -home; line -clear
   move -up 2; line -clear

   screen -fore $highlight_color
   echo -center 'Note - Database is full. Cannot add record.'
   move -down 2;

   # Prompt, and wait for acknowledgement of the error.

   echo -center '[ ANY KEY TO CONTINUE ]'
   read -key $option

   move -home; line -clear
   move -up 2; line -clear

   # Error was handled. Return.

   return

endif

# Increment the record count, and insert a record.

set $record_count + 1
set $this_record  + 1
renum -insert $static:dbase:$this_record

# Create a blank record.

call init_record

# Assign the record creation date.

set $static:dbase:$this_record:created = $date

# Display and edit this new blank record.

screen -buffer on
call update_form
screen -flush
screen -buffer off

call edit_record

# See if the user committed the changes, or escaped from the form.

if $input:string eq "esc"

   # Abandon changes:

   unset $static:dbase:$this_record
   renumber $static:dbase
   set $this_record  - 1
   set $record_count - 1

   if $this_record le 0
      set $this_record  = 1    # Clip at minimum.
   endif

endif

# Record has been added. Return

return

# # # # # # # # # # # # # # # #

callpoint: delete_record

# Delete this record as follows.

# # # # # # # # # # # # # # # #

if $record_count eq 0
   return   # Bypass. No records to delete.
endif

# Prompt for confirmation.

screen -fore $highlight_color
move -home; line -clear
echo -center "DELETE THIS RECORD? (Y/N)"
screen -fore $text_color
read -key $option

if $option eq 'y' or $option eq 'Y'

   # Operator confirmed.

   unset $static:dbase:$this_record
   renumber $static:dbase

   set $record_count - 1
   if $record_count eq 0

      # No records exist to be deleted.

      set $this_record = 1
      call init_record

   else if $this_record gt $record_count

      # Display previous record before returning.

      if $this_record gt 1
         set $this_record - 1
      endif

   endif

endif

line -clear

# Record has been deleted. Return.

return

# # # # # # # # # # # # # # # #

callpoint: edit_record

# Edit the current database record.

# # # # # # # # # # # # # # # #

# Display edit legend at the top of the screen.

screen -fore $highlight_color
move 1 1
line -clear
echo -center ~
"Up=Prev Field        Down=Next Field        Esc=Abandon        F10=End Edit"
line -invert 80

# Clear the bottom legend.

move -home ; line -clear

# Create working copy for changes.
# All changes are applied to the '$tmp' view until the user commits
# changes to the database.

loop for $tmp:i = 1 to 9
   set $tmp:($tmp:i) = $static:dbase:$this_record:($tmp:i)
endloop

# Initial parameters are as follows:

set $ycoord       = 3
set $this_field   = 1
screen -fore $text_color
unset $input:string

# Loop on edit commands.

loop

   # Move to the operant input field.

   move [ $field:xorigin - 1 ] $ycoord
   screen -fore $text_color
   screen -back $background_color
   echo -screen ' '; # Clear line margin.

   # Invert the text at this field.
   # (The line -invert is necessary to highlight the record on
   # monochrome displays.)

   screen -foreground $background_color
   screen -background $text_color
   line -invert [ $field:width + 1 ]

   # Read an input value.
   # The program stops until an input record is read.

   move -right 1
   read [ "$field:format" ] $input:string

   # Input now resides in '$input:string'. Restore field colors.

   move -left 1
   line -invert [ $field:width + 1]
   screen -fore $text_color
   screen -back $background_color

   # Process the line.

   if $input:string eq "del"

      # The input is the "Delete" function key. Delete the contents
      # of this field.

      set $tmp:$this_field = ''
      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field

   else if -test $input:string eq "f10"

      # The input is the "End Edit" function key. Redisplay the
      # current field with no changes, and break out of the command
      # loop.

      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field
      break

   else if -test $input:string eq "up"

      # The input is the "Up" function key. Redisplay the current field
      # with no changes, and reposition cursor to previous field.

      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field

      if $this_field gt 1
          set $this_field - 1
          set $ycoord - 2
      else
          set $this_field = 9
          set $ycoord = 19
      endif

   else if -test $input:string eq "esc"

      # The input is the "Abandon Changes" function key. Redisplay the
      # current field with no changes, and then query for confirmation
      # to abandon all changes.

      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field

      # Prompt as follows:

      screen -fore $highlight_color
      move -home; echo -center "ABANDON CHANGES - CONFIRM? (Y/N)"
      screen -fore $text_color
      read -key $option

      if $option eq 'y' or $option eq 'Y';

         # Abandon changes and return to the calling program.

         unset $tmp
         return

      else

         # Do not abandon changes. Clear above prompt and continue.

         line -clear

      endif

   else if -test $input:string is null or $input:string eq "down"

      # The input is either null (i.e. no change) or the "Down" function
      # key. Redisplay the current field with no changes, and reposition
      # cursor to the next field.

      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field

      if $this_field lt 9
         set $this_field + 1
         set $ycoord + 2
      else
          set $this_field = 1
          set $ycoord = 3
      endif

   else if -test $input:string is not key

      # The input string is valid. Record and display the new value.
      # When finished, position the cursor to the next field.

      move $field:xorigin $ycoord

      # Convert to upper case.

      set -inline $tmp:$this_field = echo -upper $input:string
      echo $field:format $tmp:$this_field

      if $this_field lt 9
         set $this_field + 1
         set $ycoord + 2
      else
         set $this_field = 1
         set $ycoord = 3
      endif

   else

      # This is a function key. Restore the line and reposition the
      # cursor to the next field.

      move $field:xorigin $ycoord
      echo $field:format $tmp:$this_field

      if $this_field lt 9
         set $this_field + 1
         set $ycoord + 2
      else
         set $this_field = 1
         set $ycoord = 3
      endif

   endif

endloop

# Save the changes:

set $static:dbase:$this_record:modified  = $date

loop for $tmp:i = 1 to 9
   set $static:dbase:$this_record:($tmp:i) = $tmp:($tmp:i)
endloop

if $record_count eq 0
  set $record_count = 1
  set $static:dbase:$this_record:created   = $date
endif

# Data record has been edited. Cleanup and return.

unset $tmp

return

# # # # # # # # # # # # # # #

callpoint: dial_number

# Dial a number using a Hayes compatible modem.

# # # # # # # # # # # # # # #

# Loop on command input:

loop

   screen -buffer on
   call draw_form
   call update_form

   # Display legend at top of the screen.

   move 1 1
   screen -fore $highlight_color
   line -clear  80
   echo -center "Phone Dialer Mode"
   line -invert 80

   # Clear menu region.

   screen -fore $box_color
   box -clear 1 11 80 25
   move 1 10; line

   # Display second legend at top of menu.

   screen -fore $highlight_color
   move 1 11
   line -clear 80
   echo -center "Communications Port: $static:comm:port"
   line -invert 80

   # Draw menu box.

   screen -fore $box_color
   box -style french
   box 1  12 80 11

   # Display menu options:

   screen -fore $label_color
   move 20 14 ; echo -screen "   F1   "; line -invert 8
   screen -fore $text_color
   move -right 15 ; echo -screen "Select Communications Port."

   screen -fore $label_color
   move 20 16 ; echo -screen "   F2   "; line -invert 8
   screen -fore $text_color
   move -right 15 ; echo -screen "Hangup Dialer."

   screen -fore $label_color
   move 20 18 ; echo -screen "   F5   "; line -invert 8
   screen -fore $text_color
   move -right 15 ; echo -screen "Dial Number."

   screen -fore $label_color
   move 20 20 ; echo -screen "  F10   "; line -invert 8
   screen -fore $text_color
   move -right 15 ; echo -screen "Return to Main Screen."

   screen -fore $highlight_color
   move -home; line -clear
   echo -center '[ ENTER - KEY ]'
   line -invert

   screen -flush
   screen -buffer off

   # Get a key entry.

   if [ "COM" & $static:comm:port ] is not file
      set $option = f1
   else
      read -key $option
   endif

   if $option eq f10         # Return to main screen.

      screen -buffer on
      call draw_form
      call update_form
      screen -buffer off
      screen -flush
      break

   else if $option eq f2     # Hangup the modem.

      if -cmd

         startblock

         line -clear
         echo -center '[ HANGUP...]'
         echo "+++"  > [ "COM" & $static:comm:port ]

         endblock

         sleep 1000
         break -next

      else

         # Invalid device. Handle the error.

         screen -fore $text_color
         line -clear; echo -center '!? - Invalid Communications Port.'
         screen -fore $highlight_color
         sleep 1000; break -next

      endif

   else if $option eq f5     # Dial the number:

      if -cmd

         startblock

         line -clear
         echo -center '[ DIALING...]'
         set $phone_number = $static:dbase:$this_record:4
         echo M0ATDT $phone_number > [ "COM" & $static:comm:port ]

         endblock

         sleep [ $static:comm:timeout * 1000 ]
         echo "+++"                > [ "COM" & $static:comm:port ]
         break -next


      else

         # Invalid device. Handle the error.

         screen -fore $text_color
         line -clear; echo -center '!? - Invalid Communications Port.'
         screen -fore $highlight_color
         sleep 1000; break -next

      endif

   else if $option eq f1     # Specify a comm port:

      line -clear
      echo -center 'ENTER - COMM PORT NUMBER? (1-4)? '
      read -key $option

      if $option is not integer

         screen -fore $text_color
         line -clear; echo -center '!? - Illegal entry'
         screen -fore $highlight_color
         sleep 1000; break -next

      else if $option lt 0 or $option gt 4

         screen -fore $text_color
         line -clear; echo -center '!? - Out of range'
         screen -fore $highlight_color
         sleep 1000; break -next

      else

         set $static:comm:port = $option
         set $data_changed = true

      endif

   else                  # Unsupported function key.

      screen -fore $text_color
      line -clear; echo -center '!? - Illegal function key'
      screen -fore $highlight_color
      sleep 1000; break -next

   endif

endloop

# User pressed F10 to return to main screen.

return

# # # # # # # # # # # # # # #

callpoint: sort_records

# Sort the database by a particular field.

# # # # # # # # # # # # # # #

# Initial values on entry:

screen -save
screen -clear

# Display the sort option menu:

move 1 1 ;
screen -fore $highlight_color
line -clear 80
move 11 1
echo -center ' SORT DATA RECORDS '
line -invert 59

box -style french
screen -fore $box_color
box  11 2 59 19

# Menu options are as follows:

screen -fore $label_color
move 22 4 ; echo -screen "   F1   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by Last Name."

screen -fore $label_color
move 22 6 ; echo -screen "   F2   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by City."

screen -fore $label_color
move 22 8 ; echo -screen "   F3   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by State."

screen -fore $label_color
move 22 10 ; echo -screen "   F4   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by ZIP code."

screen -fore $label_color
move 22 12 ; echo -screen "   F5   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by Date Created."

screen -fore $label_color
move 22 14 ; echo -screen "   F6   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Sort by Date Modified."

screen -fore $label_color
move 22 16 ; echo -screen "  F10   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Return to Main Screen."

screen -fore $highlight_color
move 1 22; line -clear
echo -center '[ ENTER - KEY ]'

# Get a function key.

loop

   read -key $key
   move -down 2; line -clear
   move -up   2

   if      $key eq f1    # Sort by last name.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $1
      sleep 500
      break

   else if $key eq f2   # Sort by city.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $6
      sleep 500
      break

   else if $key eq f3   # Sort by state.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $7
      sleep 500
      break

   else if $key eq f4   # Sort by ZIP.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $8
      sleep 500
      break

   else if $key eq f5   # Sort by date created.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $created
      sleep 500
      break

   else if $key eq f6   # Sort by date modified.

      line -clear
      echo -center '[ SORTING...]'
      renum -sort $static:dbase by $modified
      sleep 500
      break

   else if $key eq f10  # Return to main menu.

      screen -flush
      return

   else                 # Unsupported function key.

      screen -fore $text_color
      move -down 2;
      line -clear; echo -center '!? - Illegal function key'
      move -up 2
      screen -fore $highlight_color

   endif

endloop

screen -fore $highlight_color
line -clear
echo -center '[ DONE - ANY KEY TO CONTINUE ]'
read -key $key

# Display the first record and return.

set $this_record = 1
screen -buffer on
call draw_form
call update_form
screen -buffer off
screen -flush

# Sorting is finished. Return

return

# # # # # # # # # # # # # # #

callpoint: find_record

# Find a particular database a particular field.

# # # # # # # # # # # # # # #

screen -save
screen -clear

# Display the find option menu:

move 1 1
screen -fore $highlight_color
line -clear 80
move 11 1
echo -center ' FIND DATA RECORD '
line -invert 59

box -style french
screen -fore $box_color
box  11 2 59 19

# Menu options are as follows:

screen -fore $label_color
move 22 4 ; echo -screen "   F1   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by Last Name."

screen -fore $label_color
move 22 6 ; echo -screen "   F2   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by First Name."

screen -fore $label_color
move 22 8 ; echo -screen "   F3   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by City."

screen -fore $label_color
move 22 10; echo -screen "   F4   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by State."

screen -fore $label_color
move 22 12; echo -screen "   F5   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by ZIP code."

screen -fore $label_color
move 22 14; echo -screen "   F6   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Find Record by Comment Keyword."

screen -fore $label_color
move 22 16; echo -screen "   F9   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Repeat Last Search."

screen -fore $label_color
move 22 18; echo -screen "  F10   "; line -invert 8
screen -fore $text_color
move -right 12 ; echo -screen "Return to Main Screen."

screen -fore $highlight_color
move 1 22; line -clear
echo -center '[ ENTER - KEY ]'

# Get a function key.

loop

   read -key $key
   move -down 2; line -clear
   move 11 22

   if      $key eq f9   # Repeat search.

      if $index^value is null or $keyword^value is null

         screen -fore $text_color
         move -down 2;
         line -clear; echo -center '!? - No previous search'
         move -up 2
         screen -fore $highlight_color

      else

         set -inline $this_record = call find_keyword $index
         break

      endif

   else if $key eq f1   # Find last name.

      unset $keyword
      line -clear; echo -screen "Find Last Name."
      set $index = 1
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f2   # Find first name.

      unset $keyword
      line -clear; echo -screen "Find First Name."
      set $index = 2
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f3   # Find city.

      unset $keyword
      line -clear; echo -screen "Find City."
      set $index = 6
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f4   # Find state.

      unset $keyword
      line -clear; echo -screen "Find State."
      set $index = 7
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f5   # Find ZIP code.

      unset $keyword
      line -clear; echo -screen "Find ZIP code."
      set $index = 8
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f6   # Find comment.

      unset $keyword
      line -clear; echo -screen "Find Comment Keyword."
      set $index = 9
      set -inline $this_record = call find_keyword $index
      break

   else if $key eq f10  # Return to main menu.

      screen -flush
      return

   else                 # Unsupported function key.

      screen -fore $text_color
      move -down 2;
      line -clear; echo -center '!? - Illegal function key'
      move -up 2
      screen -fore $highlight_color

   endif

endloop

# The find operation is finished, and the value of '$this_record' has
# been modified to display the target record. Display this record and
# return.

screen -buffer on
call draw_form
call update_form
screen -buffer off
screen -flush

return

# # # # # # # # # # # # # # #

callpoint: find_keyword

# Query and find a keyword for the given record.

# # # # # # # # # # # # # # #

if $keyword^value is null

   move 11 23
   line -clear
   echo -screen 'ENTER - KEYWORD TO FIND? '

   screen -fore $box_color
   box -style single
   box  37 22 33 3

   screen -fore $text_color
   move -right 28
   echo -screen '>'
   read -scr-28 $input:string

   if $input:string is key
      echo $this_record
      return
   else 

      # Convert to upper case.

      set -inline $input:string = echo -upper $input:string
      set $keyword = $input:string

   endif

else # Repeat search

   set $input:string = $keyword

endif

set $next_record = $this_record

# Scan records for the keyword.

loop

   if $next_record ge $record_count
      set $next_record = 1
   else
      set $next_record + 1
   endif

while $next_record ne $this_record

   if $input:string in $static:dbase:$next_record:($arg:1)
      echo $next_record
      return
   endif

endloop

# The scan is finished.

if $input:string in $static:dbase:$this_record:($arg:1)

   echo $this_record
   return

endif

# No match was found.

box -style clear
box 1 22 80 25

move 11 22
screen -fore $highlight_color
echo -center '[ KEYWORD NOT FOUND - ANY KEY TO CONTINUE ]'
read -key $key

echo $this_record
return

# # # # # # # # # # # # # # #

callpoint: display_help_screen

# Display the help screens.

# # # # # # # # # # # # # # #

# Save the current display.

screen -save
screen -clear

# Display legend at the top of the screen.

screen -fore $highlight_color
move 1 1;
line -clear
echo -center ~
"   (Any Key for More Help ...)                        F10=To Exit Help"
line -invert

# Display the first screen:

screen -fore $box_color
box -style french
box 1 2 80 21
screen -fore $highlight_color
move 8 4; echo -screen "HOW TO USE THE ADDR PROGRAM"

screen -fore $text_color
move -down  1

echoblock -scr

The "ADDR.S" script is a simple address database manager that
allows address data to be added, edited, deleted, sorted, or
searched. The "ADDR.S" script additionally includes a phone
dialer function that works with Hayes compatible modems.

Address data is contained in the file "addr.dat" in the directory
where the program executes, or in the file specified by "$dbfile".
The "addr.dat" file can be manually edited, and can be sourced
by other CIEX scripts which rely on address information.

Currently, the "ADDR.S" script accepts a maximum of 50 address
entries, although this limit can be increased by manipulating the
value of "$max_records" defined in the script file.

endblock

# Wait for a key entry.

read -key $option
if -test $option eq f10
   screen -flush; return
endif

# Display the second screen:

box -clear 2 4
screen -fore $highlight_color
move 8 4; echo -screen "ENTERING DATA RECORDS"

screen -fore $text_color
move -down  1

echoblock -scr

The "ADDR.S" program starts in "viewing" mode, where data records
can be viewed but not changed. To enter "edit" mode (to add or change
data records), select the "F2" or the "F3" key.

Once the user is in "edit" mode, fields can be entered, modified, or
deleted. To leave the edit mode, use the "F10" key to terminate the
editing session and return to viewing mode. To escape from an edit
session without changing or adding the record, use the "Esc" key.

While in the "edit" mode, the "Up" and "Down" arrow keys will
switch fields. The "Enter" key commits changes to fields (if any.)
The "Delete" key will delete the contents of the current field.

endblock

# Wait for a key entry.

read -key $option
if -test $option eq f10
   screen -flush; return
endif

# Display the third screen:

box -clear 2 4
screen -fore $highlight_color
move 8 4; echo -screen "VIEWING DATA RECORDS"

screen -fore $text_color
move -down  1

echoblock -scr

Record viewing keys are listed at the bottom of the screen. Use the
"PgDn" key to go to the next record. Use the "PgUp" key to go to the
previous record. The "Home" and "End" keys view the first and last
records respectively.

To search for a particular record by a field value, use the "F9" key
to display the "Find" option menu. Records can be searched for using
the standard CIEX wildcards (*, ?, +, ^, and $). Note that search
words are not case sensitive.

To support data viewing, data records can be sorted by selected
fields using the "F8" key. Sorting is not very fast, however
affords a fair amount of functionality, including the ability to sort
by the dates on which records were created or modified.

endblock

# Wait for a key entry.

read -key $option
if -test $option eq f10
   screen -flush; return
endif

# Display the fourth screen:

box -clear 2 4
screen -fore $highlight_color
move 8 4; echo -screen "PHONE DIALER FUNCTION"

screen -fore $text_color
move -down  1

echoblock -scr

To use the automatic phone dialer, use the "F5" key to display the
"Dialer" option menu. This menu allows parameters to be selected
(including the COMM port), and allows a number to be automatically
dialed.

The "Dialer" function requires a Hayes compatible modem to be
correctly installed and connected to a phone line. To use the
dialer, select a record and press the "F5" key twice. The phone
number of the current record (if any) will be automatically dialed.
(If the phone is not off-hook when the dialing occurs, the phone
will automatically hang-up after a few seconds.)

Note that if the modem is not correctly installed (or the dialer is
misconfigured) various system error messages are possible.

endblock

# Wait for a key entry.

read -key $option
if -test $option eq f10
   screen -flush; return
endif

# Display the fifth screen:

box -clear 2 4
screen -fore $highlight_color
move 8 4; echo -screen "FUNCTION KEYS DEFINITIONS"

screen -fore $text_color
move -down  1

echoblock -scr

   [ F1=Help    ]     Display this help message.
   [ F2=Add     ]     Add a record to the address database.
   [ F3=Edit    ]     Edit the currently displayed record.
   [ F4=Del     ]     Delete the currently displayed record.
   [ F5=Dial    ]     Display the phone dialer options screen.
   [ F8=Sort    ]     Display the "sort" menu to sort records.
   [ F9=Find    ]     Display the "find" menu to find a record.
   [ F10=Exit   ]     Exit the program.

   [ PgUp       ]     Display the previous record.
   [ PgDn       ]     Display the next record.
   [ Home       ]     Display the first record.
   [ End        ]     Display the last record.

endblock

# The help facility is done.

move -home; move -up
screen -fore $highlight_color
echo -center '[ ANY KEY TO RETURN TO MAIN SCREEN ]'
read -key $option
screen -flush
return

# # # # # # # # # # # # # # #

callpoint: init_record

# The data record is as follows:

# # # # # # # # # # # # # # #

set $static:dbase:$this_record:1 = ''
set $static:dbase:$this_record:2 = ''
set $static:dbase:$this_record:3 = ''
set $static:dbase:$this_record:4 = ''
set $static:dbase:$this_record:5 = ''
set $static:dbase:$this_record:6 = ''
set $static:dbase:$this_record:7 = ''
set $static:dbase:$this_record:8 = ''
set $static:dbase:$this_record:9 = ''

set $static:dbase:$this_record:created   = '0000-00-00'
set $static:dbase:$this_record:modified  = '0000-00-00'

# # # # # # # # # # # # # # # # # # # # # #

callpoint: interrupt

# # # # # # # # # # # # # # # # # # # # # #

call -goto end_session

# # # # # # # # # # # # # # # #

callpoint: end_session

# Save the database and exit.

# # # # # # # # # # # # # # # #

if $data_changed^value ne true

   # Data was not modified. Exit.

   return -entry

endif

# Data was modified. Query to save the data.

move -home
line -clear
screen -fore $highlight_color
echo -center '[ DATA CHANGED - SAVE? (Y/N) ]'
read -key $key

if $key eq 'y' or $key eq 'Y'

   # Save the data changes and exit.

   if -cmd

      list -all $static > $dbfile
      return -entry

   else

      # Could not save the database.

      scr -buffer off
      scr -def
      scr -clear
      move 1 15
      move -down; echo -scr "ERROR: Cannot create address info file..."
      move -down; echo -scr "$sys:errtxt $sys:errloc"
      move -down; echo -scr "Is local disk write protected?"
      move -down; echo -scr "Press any key to continue..."
      read -key $confirm
      return -entry  # End the program.

   endif

else if $key eq 'n' or $key eq 'N'

   # Exit without saving the data changes.

   return -entry

else

   # Operator did not confirm exit. Update the display and return.

   screen -buffer on
   call draw_form
   call update_form
   screen -buffer off
   screen -flush
   return

endif

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the addr.s program.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: addr.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"
move -down; echo -scr "Press any key to continue..."
read -key $arg:exit

return -entry

