# # # # # # # # # # # # # # # # # # # # # # #

# CALENDAR.S - Desk Top Calendar.
# CIEX (C) 1993, 1994 New Line Software, Inc.
# Script version 1.2

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Execute a calendar program with appointment database.

# # # # # # # # # # # # # # # # # # # # # # #

callpoint: top

# Initialize the script:

trace  -off
echo -off
unset -calls
source -calls

# Initialize the environment.

unset  -view
unset  -all

# These are the colors used by the program:

set $highlight_color  = cyan
set $label_color      = green
set $box_color        = red
set $text_color       = white
set $background_color = black

# Location of the database file is:

if \ciex\data is directory
   set $appointment_book = "\CIEX\DATA\APPT.DAT"
else
   set $appointment_book = "APPT.DAT"
endif

# Restore the appointment database if it exists.

if $appointment_book is file
   setblock -file $appt = $appointment_book
endif

# Initialize calendar constants:

call -view calendar_constants

# Get today's date.

setblock -cmd = call -view todays_date

# Begin buffering the screen output.

screen -default
screen -buffer on

# Initialize the calendar display.

call initialize_calendar

# Update the calendar with the days of the month.

call update_calendar

# The calendar is now being displayed. Loop to get keyboard input:

loop

   # Program stops until user inputs a key:

   scr -flush
   read -key $key

   # Decode the value of the key:

   if      $key eq right ; call next_day
   else if $key eq left  ; call prev_day
   else if $key eq down  ; call next_week
   else if $key eq up    ; call prev_week

   else if $key eq f1    ; call display_cal_help_screen
   else if $key eq f3    ; call prev_month
   else if $key eq f4    ; call next_month
   else if $key eq f5

      # Run the utility menu.

      call run_utility_menu

      # Utility menu is finished. Redisplay the calendar.

      call initialize_calendar
      call update_calendar

   else if $key eq enter

      # Run the appointment book screen.

      call run_appts

      # Appointment screen is finished. Redisplay the calendar.

      call initialize_calendar
      call update_calendar

   else if $key eq home

      # Reset calendar to today's date.

      setblock -cmd = call -view todays_date
      call update_calendar

   else if $key eq f10

      # Exit this program.

      scr -buffer off
      scr -clear
      call -goto exit

   endif

endloop

# Exit the program.

call -goto exit
return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: initialize_calendar

# Draw the calendar grid on the display.

# # # # # # # # # # # # # # # # # # # # # #

select -curs off
scr -back black
scr -fore $label_color
box -style single
box 1 2 80 22

# Draw the top control bar.

screen -fore $highlight_color
screen -back $background_color

move 1 1
line -clear 80
echo -center ~
"F1=Help       F3=Prev Month       F4=Next Month       F5=Utils       F10=Exit"
line -invert 80

# Draw the bottom control bar.

move -home
line -clear 80
line -invert 80

# Draw the calendar outline

screen -fore $box_color
screen -back $background_color
box -style french
box  11 4 60 18

# Create field for todays date.

move 16 4 ; line -clear
move 16 4 ; line -invert

# Display the days of the week.

screen -fore $label_color
move 13 8
echo -scr "  Sun     Mon     Tue     Wed     Thu     Fri     Sat  "

# Cross-hatch lines.

screen -fore $box_color
move 12 7;     line -horiz
move -down 2;  line -horiz
move -down 2;  line -horiz
move -down 2;  line -horiz
move -down 2;  line -horiz
move -down 2;  line -horiz
move -down 2;  line -horiz

move 20 7;     line -vert 15
move -right 8; line -vert 15
move -right 8; line -vert 15
move -right 8; line -vert 15
move -right 8; line -vert 15
move -right 8; line -vert 15

# The calendar outline has been created. Flush the screen and return.

scr -flush

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: update_calendar

# Update the calendar with the days of the month. The screen is buffered
# for a more aesthetic appearance.

# # # # # # # # # # # # # # # # # # # # # #

scr -fore $highlight_color
move 18 6 ; line -clear 45  # Clear annual message from the screen.

# Determine the day of the week on which this month starts.

call get_start_weekday

# Get the size of this month:

set $size = $calendar_constants:$month:size

if $size eq 28

   if -eval ~
      ($yr % 4) eq 0 and (($yr % 100 ne 0) or ($yr % 400 eq 0)))
      set $size + 1 # Leap year
   endif

endif

# Initialize the foreground color and coordinates:

screen -fore $text_color
screen -back $background_color

set $x = 13
set $y = 10

# Initialize start of the month:

set $day = 0

# Clear the first few days of the month from the screen.

move $x $y
loop while $day lt $start_wkday

   line -clear 6
   move -right 8
   set $x + 8
   set $day + 1

endloop

# Update calendar with the days of the month.

loop for $day = 1 to $size

   echo -scr6 "$day "; set $x + 8

   if $x gt 66 # Next week
      move -down 2; set $y + 2; set $x = 13
   endif

   move $x $y

endloop

# Clear the last few days at the end of the month.

loop while $y lt 21

   line -clear 6; move -right 8; set $x + 8

   if $x gt 66 # Next week

      move -down 2; set $y + 2; set $x = 13
      move $x $y

   endif

endloop

# Update the date at the top of the screen:

screen -fore $box_color

move 16 4 ; line -clear
move 24 4 ; echo -scr-12 "$calendar_constants:$month:name"
move 39 4 ; echo -scr2   "$today"
move 47 4 ; echo -scr10  "$yr"
move 16 4 ; line -invert

# Highlight the current day:

call mark_today

# Stop buffering, and flush the data to the screen.

# # # # # # # # # # # # # # # # # # # # # #

callpoint: next_day

# Move the highlight bullet to tomorrow on the calendar.
# This callpoint is executed when the 'right' arrow key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5
set $today + 1
if $today gt $size
  set $today = 1
endif
call mark_today

# # # # # # # # # # # # # # # # # # # # # #

callpoint: prev_day

# Move the highlight bullet to yesterday on the calendar.
# This callpoint is executed when the 'left' arrow key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5
set $today - 1
if $today lt 1
   set $today = $size
endif
call mark_today

# # # # # # # # # # # # # # # # # # # # # #

callpoint: next_week

# Move the highlight bullet to the next week on the calendar.
# This callpoint is executed when the 'down' arrow key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5
set $today + 7
if $today gt $size
   set $today % 7
endif
if $today eq 0
   set $today + 7
endif
call mark_today

# # # # # # # # # # # # # # # # # # # # # #

callpoint: prev_week

# Move the highlight bullet to the previous week on the calendar.
# This callpoint is executed when the 'up' arrow key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5
set $today - 7
if $today lt 1
   set -eval $today = $today  + 42
   loop while $today gt $size
      set $today - 7
   endloop
endif
call mark_today

# # # # # # # # # # # # # # # # # # # # # #

callpoint: next_month

# Display next month's calendar.
# This callpoint is executed when the 'F4' key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5
if $month eq 12
   set $month = 1
   set $yr + 1
else
   set $month + 1
endif

call update_calendar

# # # # # # # # # # # # # # # # # # # # # #

callpoint: prev_month

# Display previous month's calendar.
# This callpoint is executed when the 'F3' key is pressed
# on the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

move $x $y ; line -invert 5

if $month eq 1
   set $month = 12
   set $yr - 1
else
   set $month - 1
endif

call update_calendar

# # # # # # # # # # # # # # # # # # # # # #

callpoint: mark_today

# Mark today's date with a highlight bullet, and at the top of the
# screen. Also, display number of appointments for today.

# # # # # # # # # # # # # # # # # # # # # #

set $x = 16
set $y = 10

# If today's date is greater than the last day of this month, set the
# date to the last day of this month. This is necessary, for example,
# on January 31st when flipping to February with the "Next Month"
# key:

if $today gt $size
   set $today = $size
endif

# Update the day at the top of the screen.

screen -fore $box_color
move 39 4 ; echo -scr2   "$today"; line -invert 2

# Determine x/y coordinates of new day on calendar screen.

set -eval $x = $x - 2 + (($today + $start_wkday - 1) % 7) * 8)
set -eval $y = $y + int( ($today + $start_wkday - 1) / 7) * 2

# Highlight today's date on the calendar

move $x  $y ; line -invert 5

# Display the annual message, if any:

screen -fore $highlight_color
move 18 6 ; line -clear 45
if $calendar_constants:annual:$month:$today^value is not null
   echo -center $calendar_constants:annual:$month:$today
endif

# Display message indicating number of appointments today.

scr -fore $highlight_color
move -home; line -clear
move -right 10
set $q = $appt:$yr:$month:$today^range

if $q is null
   echo -scr "No Appointments.        "
else if $q eq 1
   echo -scr "1 Appointment Entry.    "
else
   echo -scr "$q Appointment Entries. "
endif

# Append the CIEX ID to the bottom legend.
move -right 40; echo -scr-20 "CIEX V2.3, NLS, Inc."
move -home ; line -invert 80

# # # # # # # # # # # # # # # # # # # # # #

callpoint: get_start_weekday

# Determine the day of the week that this month starts on. The routine
# returns a number from 0 to 6 in the variable $start_wkday. The value
# is used to draw a calendar for a particular month.

# # # # # # # # # # # # # # # # # # # # # #

# The algorithm is as follows:

set -eval $start_wkday = ~
   $calendar_constants:$month:factor + $yr + 1

set -eval $start_wkday = ~
   $start_wkday + int($yr/4) + int($yr/400) - int($yr/100)

if $month lt 3

   if -eval ~
      ($yr % 4) eq 0 and (($yr % 100 ne 0) or ($yr % 400 eq 0)))
      set $start_wkday - 1
   endif

endif

set $start_wkday % 7

# # # # # # # # # # # # # # # # # # # # # #

callpoint: run_appts

# Run the appointment book screen. This callpoint is executed when the
# 'Enter' key is pressed from the main calendar display.

# # # # # # # # # # # # # # # # # # # # # #

# Initialize the appointment book and update the display.

call initialize_appointment_book
call update_appointments

# The appointment book is being displayed. Get the appointment data:

loop

   # Generate a prompt, and highlight the operant line.

   move $x $y       # Move to start of this field.
   echo -scr '>'    # Display a prompt.
   move -right 2    # Move over 2 columns.
   line -invert 60  # Highlight this line.
   screen -flush    # Show the display.

   # Get a line from the user. Program stops until line is entered.

   read -scr60 $keyboard_input
   move -left 2
   echo -scr ' '

   # A line has been entered. Process the line.

   if $keyboard_input eq "f1"   # Call the help screens.

      call display_appt_help_screen

      move $x $y
      line -clear 65
      if $appt:$yr:$month:$today:$hour^value is not null
         move -right 2
         echo -scr $appt:$yr:$month:$today:$hour
         move -left 2
      endif

   else if $keyboard_input eq "f3" # Display yesterdays appts

      call set_yesterday
      call update_appointments

   else if $keyboard_input eq "f4"   # Display tomorrows appts

      call set_tomorrow
      call update_appointments

   else if $keyboard_input eq "f10"  # Return to calendar

      return

   else if $keyboard_input eq del    # Delete this appointment.

      unset $appt:$yr:$month:$today:$hour
      line -clear 65

   else if $keyboard_input eq "up" or $keyboard_input eq "pgup"

      # Scroll up:

      line -clear 65
      if $appt:$yr:$month:$today:$hour^value is not null
         move -right 2
         echo -scr $appt:$yr:$month:$today:$hour
         move -left 2
      endif

      set $y - 2
      set $hour - 2
      move $x $y

   else if $keyboard_input is key or $keyboard_input is null

      # Scroll down

      line -clear 65
      if $appt:$yr:$month:$today:$hour^value is not null
         move -right 2
         echo -scr $appt:$yr:$month:$today:$hour
         move -left 2
      endif

      set $y + 2
      set $hour + 2
      move $x $y

   else if '^  +$' in $keyboard_input  # Delete this appointment.

      unset $appt:$yr:$month:$today:$hour
      line -clear 65
      set $y + 2
      set $hour + 2
      move $x $y

   else  # Store this appointment.

      set -replace $keyboard_input = '^ +'
      set -replace $keyboard_input = ' +$'
      set $appt:$yr:$month:$today:$hour = $keyboard_input
      set $y + 2
      set $hour + 2
      move $x $y

   endif

   # Roll over y coordinate as needed.

   if $y gt 23

      set $x = 16; set $y = 5; set $hour = 4
      move $x $y

   else if $y lt 5

      set $x = 16; set $y = 23; set $hour = 22
      move $x $y

   endif

endloop

return

# # # # # # # # # # # # # # #

callpoint: set_yesterday

# Set the date to yesterday. This callpoint is executed when the
# F3 key is pressed in the appointment book screen.

# # # # # # # # # # # # # # #

set $today - 1

if $today gt 0
   return
endif

if $month eq 1 # Roll over year

   set $size  = 31
   set $month = 12
   set $today = 31
   set $yr - 1;
   return

else  # Roll over month

   set $month - 1

endif

set $size = $calendar_constants:$month:size

if $size eq 28

   if -eval ~
      ($yr % 4) eq 0 and (($yr % 100 ne 0) or ($yr % 400 eq 0)))
      set $size + 1 # Leap year
   endif

endif

set $today = $size
return

# # # # # # # # # # # # # # #

callpoint: set_tomorrow

# Set the date to tomorrow. This callpoint is executed when the
# F4 key is pressed in the appointment book screen.

# # # # # # # # # # # # # # #

set $today + 1

if $today le $size
   return
endif

if $month ge 12 # Roll over year

   set $size  = 31
   set $month = 1
   set $today = 1
   set $yr + 1;
   return

else  # Roll over month

   set $month + 1
   set $today = 1

endif

set $size = $calendar_constants:$month:size

if $size eq 28

   if -eval ~
      ($yr % 4) eq 0 and (($yr % 100 ne 0) or ($yr % 400 eq 0)))
      set $size + 1 # Leap year
   endif

endif
return

# # # # # # # # # # # # # # #

callpoint: initialize_appointment_book

# Draw the appointment screen.

# # # # # # # # # # # # # # #

screen -clear

move 1 1
screen -fore $highlight_color

line -clear 80
echo -center ~
"F1=Help            F3=Prev Day            F4=Next Day            F10=Return"
line -invert 80

scr -fore $label_color
move 1 3
move -down 2; echo -scr12  "COMMENT"
move -down 2; echo -scr12  "6:00 AM"
move -down 2; echo -scr12  "8:00 AM"
move -down 2; echo -scr12 "10:00 AM"
move -down 2; echo -scr12 "12:00 PM"
move -down 2; echo -scr12  "2:00 PM"
move -down 2; echo -scr12  "4:00 PM"
move -down 2; echo -scr12  "6:00 PM"
move -down 2; echo -scr12  "8:00 PM"
move -down 2; echo -scr12 "10:00 PM"

screen -fore $box_color
move 2 4; line

loop for $i = 1 to 10
   move -down 2; line;
endloop

move 14 4  ; line -vert 21

# # # # # # # # # # # # # # #

callpoint: update_appointments

# Update the appointment book with today's appointments.

# # # # # # # # # # # # # # #

screen -fore $highlight_color
move 16 3; echo -scr-45 ""

move 16 3
echo -scr-45 "$calendar_constants:wkday:[($today + $start_wkday - 1) % 7]"
move 27 3 ; echo -scr "$calendar_constants:$month:name  "
move 37 3 ; echo -scr "$today, "
move 42 3 ; echo -scr "$yr"

screen -fore $text_color

set $x = 16 ; set $y = 5; set $hour = 4
move $x $y

loop for $i = $y to 23 by 2

   move $x $i
   line -clear 80
   if  $appt:$yr:$month:$today:$hour^value is not null
      echo -scr "  $appt:$yr:$month:$today:$hour"
   endif
   set $hour + 2

endloop

set $x = 16 ; set $y = 5; set $hour = 4
move $x $y

# # # # # # # # # # # # # # # # # # # # # #

callpoint: todays_date

# Get the current BIOS date.
# Usage: set -inline $var = call todays_date

# # # # # # # # # # # # # # # # # # # # # #

unset $todays_date
set -inline $todays_date = startblock
   echo |date
endblock

if $todays_date is null

   echo month 1
   echo today 1
   echo yr    1994
   return

endif

set -list $todays_date = $todays_date
set $todays_date = $todays_date:5
set -list $todays_date = $todays_date by '-'

echo month [ $todays_date:1 ]
echo today [ $todays_date:2 ]
echo yr    [ $todays_date:3 ]

# # # # # # # # # # # # # # #

callpoint: run_utility_menu

# Run the utilities menu:

# # # # # # # # # # # # # # #

screen -clear
screen -fore $highlight_color

# Display the day at the top of the screen:

set $disp_string = ""
set $disp_string ~
   & "$calendar_constants:wkday:[($today + $start_wkday - 1) % 7],  "
set $disp_string & "$calendar_constants:$month:name  "
set $disp_string & "$today,  "
set $disp_string & "$yr"

move 10  2; line -clear; echo -center $disp_string
line -invert

screen -fore $box_color

box -style french
box  10 4 62 16

screen -fore $label_color
move 15 7;  echo -scr "  F1   "; line -invert 7
scr -fore $text_color
move -right 10 ; echo -scr "Set calendar to arbitrary month and year."

scr -fore $label_color
move 15 9;  echo -scr "  F2   "; line -invert 7
scr -fore $text_color
move -right 10 ; echo -scr "Purge appointments prior to this month."

scr -fore $label_color
move 15 11; echo -scr "  F3   "; line -invert 7
scr -fore $text_color
move -right 10 ; echo -scr "List all appointments to a file."

scr -fore $label_color
move 15 13; echo -scr "  F9   "; line -invert 7
scr -fore $text_color
move -right 10 ; echo -scr "Exit without saving appointments."

scr -fore $label_color
move 15 15; echo -scr "  F10  "; line -invert 7
scr -fore $text_color
move -right 10 ; echo -scr "Return to calendar."

# Get command input:

loop

   scr -fore $highlight_color
   box -style clear
   box 1 21 80 3
   move 16 22; echo -center '[ ENTER - KEY ]'

   screen -flush
   read -key $key
   move -home; line -clear

   if $key eq f1    # Set calendar to arbitrary day:

      call query_date

   else if $key eq f2 # Purge appointments prior to this month:

      move 1 22; line -clear
      echo -center '[ PURGE APPOINTMENT DATA - CONFIRM? (Y/N) ]'
      screen -flush
      read -key $key
      if $key eq 'y' or $key eq 'Y'
         call purge_appts
      endif

   else if $key eq f3    # List appointments to a file:

      call list_appts_to_file

   else if $key eq f8    # Change appointment book

      call change_appt_book

   else if $key eq f9 # Abort without saving

      move 1 22; line -clear
      echo -center '[ EXIT WITHOUT SAVING - CONFIRM? (Y/N) ]'
      screen -flush
      read -key $key
      if $key eq 'y' or $key eq 'Y'
         call -goto exit_no_save
      endif

   else if $key eq f10 # Return to calendar:

      return

   endif

endloop

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: query_date

# Query and check a date value. The date is displayed at the top of the
# menu, and returned in the variables "$month" and "$yr". The value of
# "$today" # is automatically adjusted to "1" (the first day of the
# month). The size of the month is stored in the variable $size.

# # # # # # # # # # # # # # # # # # # # # #


screen -fore $box_color
box -style single
box 55 21 15 3

screen -fore $highlight_color
move 16 22; echo -scr "~[ ENTER - MM YYYY (EXAMPLE: $month $yr) ]"

move 57 22;
screen -fore $text_color
echo -scr '>'
screen -flush
read -scr8 $keyboard_input
set -replace $keyboard_input = '/' by ' '
screen -fore $highlight_color

set -list $line = $keyboard_input

if      -test $line^range lt 1 ; return
else if -test $line^range ne 2

   screen -fore $text_color
   move -home; line -clear
   echo -center "!? - Two numbers expected. No action."
   return

else if -test $line:1 is not integer or $line:2 is not integer

   screen -fore $text_color
   move -home; line -clear
   echo -center "!? - Integer input expected. No action."
   return

else if -test $line:1 lt 1 or $line:1 gt 12

   screen -fore $text_color
   move -home; line -clear
   echo -center "!? - Month out of range. No action."
   return

else if -test $line:2^size eq 2

   if -test $line:2 lt 0 or $line:2 gt 99
      screen -fore $text_color
      move -home; line -clear
      echo -center "!? - Year out of range. No action."
      return
   else
      set $yr = 19;
      set $yr & $line:2
   endif

    set $yr = [ int $yr ]

else if -test $line:2^size eq 4

   if -test $line:2 lt 1000 or $line:2 gt 9999
      screen -fore $text_color
      move -home; line -clear
      echo -center "!? - Year out of range. No action."
      return
   else
      set $yr = [ int $line:2 ]
   endif

else

   screen -fore $text_color
   move -home; line -clear
   echo -center "!? - Year out of range. No action."
   return

endif

set $today = 1
set $month = [ int $line:1 ]
set $size = $calendar_constants:$month:size
call get_start_weekday

if $size eq 28

   if -eval ~
      ($yr % 4) eq 0 and (($yr % 100 ne 0) or ($yr % 400 eq 0)))
      set $size + 1 # Leap year
   endif

endif

set $disp_string = ""
set $disp_string ~
   & "$calendar_constants:wkday:[($today + $start_wkday - 1) % 7],  "
set $disp_string & "$calendar_constants:$month:name  "
set $disp_string & "$today,  "
set $disp_string & "$yr"

move 1  2; line -clear; echo -center $disp_string

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: purge_appts

# # # # # # # # # # # # # # # # # # # # # #

# Unset all previous years:

startblock

   set $appt:tmp:year  = $yr
   set $appt:tmp:count = 0
   set -view $appt

   loop for -cmd $tmp:entry = list

      set -list $tmp:entry = $tmp:entry
      set $tmp:entry = $tmp:entry:1^value
      if $tmp:entry is integer

         if $tmp:entry lt $tmp:year
            set $tmp:count + 1
            unset $($tmp:entry)
         else
            break
         endif

      endif

   endloop

endblock

set $count = $appt:tmp:count
unset $appt:tmp

# Unset all previous months:

startblock

   set $appt:$yr:tmp:month = $month
   set $appt:$yr:tmp:count = 0
   set -view $appt:$yr

   loop for -cmd $tmp:entry = list

      set -list $tmp:entry = $tmp:entry
      set $tmp:entry = $tmp:entry:1^value
      if $tmp:entry is integer

         if $tmp:entry lt $tmp:month
            unset $($tmp:entry)
            set $tmp:count + 1
         else
            break
         endif

      endif

   endloop

endblock

set $count + $appt:$yr:tmp:count
unset $appt:$yr:tmp
echo -scr

screen -fore $text_color
move -home; line -clear
if $count eq 0
   echo -center "Note - no previous appointments found."
else
   echo -center "Appointments purged."
endif

unset $count
return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: list_appts_to_file

# # # # # # # # # # # # # # # # # # # # # #

# Query for a filename:

move 16 22; line -clear
screen -fore $highlight_color
echo -scr "~[ FILENAME? ] "

# Create an input window as follows:

screen -fore $box_color
box -style single
box 30 21 40 3

# Generate a cursor and prompt:

move 32 22;
screen -fore $text_color
echo -scr '>'
move -right 2; echo -scr '_'

# Read the filename:

read -scr34 $keyboard_input

set $filename = $keyboard_input

# Clear the display

screen -fore $highlight_color
box -style clear
box 1 21 80 5
scr -flush

if $filename^size is null or $filename is key

   # A blank function key or return was entered.

   return

else if $filename is file

   # File already exists.

   move 1 22
   echo -center "~[ OVERWRITE EXISTING FILE? (Y/N) ] "

   scr -flush
   read -key $key
   if $key ne 'y' and $key ne 'Y'

      # Don't overwrite the existing file name.

      return

   endif

endif

# Initialize the file.

screen -fore $text_color
if -cmd ; echo "" > $filename
else

   # File name could not be created.

   move -home; line -clear
   echo -center "!? - Cannot create - $filename"
   return

endif

# List appointments to the file.

call list_appts >> $filename

# Verify the file was created.

if $filename is file
   move -home; line -clear
   echo -center "File - $filename - created."
else
   move -home; line -clear
   echo -center "!? - $filename - not created."
endif

# Appointments have been listed to a file. Return.

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: list_appts

# List all appointments to standard output.
# (The calling routine will redirect these appointments to
# a specified filename.)

# # # # # # # # # # # # # # # # # # # # # #

set -view $appt

move 1 21 ; line -clear
move 1 22 ; line -clear
move 1 23 ; line -clear

move 16 22
set $tmp:count = 0

loop for -cmd $tmp:entry = list -all

   # Break entry apart at tab. Note: hidden tab character.

   set -list $tmp:field = $tmp:entry by '	'

   # Break appointment apart at colons:

   set -list $tmp:field = $tmp:field:1 by ':'

   # See if this entry is an appointment:

   if $tmp:field^range ne 4
      break -next
   else if $tmp:field:1 is not integer or ~
      $tmp:field:2 is not integer or ~
      $tmp:field:3 is not integer
      $tmp:field:4 is not integer
      break -next
   endif

   # Display the date:

   echo -int2.2 $tmp:field:2  # Month
   echo -fmt "/"
   echo -int2.2 $tmp:field:3  # Day
   echo -fmt "/"
   echo -int4.4 $tmp:field:1  # Year
   echo -fmt " "

   # Display the hour:

   if $tmp:field:4 ne 4
      echo -int2.2 $tmp:field:4 # This is an appointed hour
      echo -fmt ":00  "
   else
      echo -fmt "-----  "       # This is an appointment comment.
   endif

   # Display the appointment message:
   # Remove characters preceding the tab mark.

   set -replace $tmp:entry = '^*	'
   echo $tmp:entry

   # Update the status line.

   screen -buffer off
   line -clear
   set $tmp:count + 1
   set -inline $tmp:count = echo -int4.4 $tmp:count
   echo -center "Appointments: $tmp:count"
   screen -buffer on

endloop

unset $tmp

# # # # # # # # # # # # # # # # # # # # # #

callpoint: display_cal_help_screen

# Display help for the calendar screen. The screen is first stored
# to memory, and then restored when the callpoint exits.

# # # # # # # # # # # # # # # # # # # # # #

screen -save
screen -buffer off
screen -clear
box -style french

screen -fore $highlight_color
move 1 1
line -clear 80
echo -center "(Any key to return to main screen...)"
line -invert

scr -fore $box_color
box 1 2 80 22

scr -fore $highlight_color
move 5 5; echo -scr "HOW TO USE THE CALENDAR PROGRAM."

scr -fore $text_color
move -down  1

echoblock -scr

Select a day using the arrow keys and press the "Enter" key to access
the appointment book. The selected day is highlighted, and the
number of appointments for the day is displayed at the bottom of the
calendar.

endblock

move -down 6
scr -fore $highlight_color
echo -scr "KEY DEFINITIONS:"
scr -fore $text_color
move -down

echoblock -scr

[ Enter         ]    - Access the appointment book for today.
[ F1=Help       ]    - Display this help message.
[ F3=Prev Month ]    - Display the previous calendar month.
[ F4=Next Month ]    - Display the next calendar month.
[ F5=Utils      ]    - Display utilities for managing appointments.
[ F10=Exit      ]    - Exit the program.
[ Home          ]    - Display the calendar for today's date.

endblock

read -key $option
screen -flush
screen -buffer on

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: display_appt_help_screen

# Display help for the appointment book screen. The screen is
# first stored to memory, and then restored when the callpoint
# exits.

# # # # # # # # # # # # # # # # # # # # # #

screen -save
screen -buffer off
screen -clear
box -style french

screen -fore $highlight_color
scr -clear
move 1 1; echo -center "(Any key to return to main screen ...)"
line -invert 80

scr -fore $box_color
box 1 2 80 22

scr -fore $highlight_color
move 5 5; echo -scr "HOW TO USE THE APPOINTMENT BOOK."

scr -fore $text_color
move -down  1

echoblock -scr

Select an entry using the "Enter", "Tab", "Down-arrow", or "Up-arrow"
keys. Then, type in an appointment. Any existing appointment entry is
replaced.

endblock

move -down 5
scr -fore $highlight_color
echo -scr "FUNCTION KEYS:"
scr -fore $text_color
move -down

echoblock -scr

[ F1=Help       ]      Display this help message.
[ F3=Prev Day   ]      Display yesterday's appointments.
[ F4=Next Day   ]      Display tomorrow's appointments.
[ F10=Return    ]      Return to the calendar screen.

endblock

read -key $option

screen -flush
screen -buffer on
return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: calendar_constants

# These are various constants used by the program.

# # # # # # # # # # # # # # # # # # # # # #

# Initialize the following:

setblock

   $1:name    January
   $1:size    31
   $1:factor  6

   $2:name    February
   $2:size    28
   $2:factor  2

   $3:name    March
   $3:size    31
   $3:factor  2

   $4:name    April
   $4:size    30
   $4:factor  5

   $5:name    May
   $5:size    31
   $5:factor  0

   $6:name    June
   $6:size    30
   $6:factor  3

   $7:name    July
   $7:size    31
   $7:factor  5

   $8:name    August
   $8:size    31
   $8:factor  1

   $9:name    September
   $9:size    30
   $9:factor  4

   $10:name   October
   $10:size   31
   $10:factor 6

   $11:name   November
   $11:size   30
   $11:factor 2

   $12:name   December
   $12:size   31
   $12:factor 4

endblock

setblock

   $wkday:0   Sunday
   $wkday:1   Monday
   $wkday:2   Tuesday
   $wkday:3   Wednesday
   $wkday:4   Thursday
   $wkday:5   Friday
   $wkday:6   Saturday

endblock

# The following are annual dates. Each date is represented by the
# month and day upon which the annual event occurs. The textual values
# are displayed at the top of the calendar when particular date is
# selected.

setblock $annual

   1:1    New Year's Day
   1:15   Martin Luther King, Jr.'s Birthday
   1:19   Robert E. Lee's Birthday
   1:20   Inauguration Day
   1:26   Indian Republic Day
   1:29   Australia Day
   2:2    Groundhog Day
   2:11   Japanese National Foundation Day
   2:12   Lincoln's Birthday
   2:14   Valentine's Day
   2:22   President's Day
   2:22   Washington's Birthday
   2:29   Leap Day!
   3:8    Women's Day
   3:17   St. Patrick's Day
   3:26   Arbor Day
   4:1    April Fools' Day
   4:7    United Nations World Health Day
   4:14   Pan American Day
   4:19   St. Joseph's Day
   5:1    May Day
   5:8    European V-E Day
   5:30   Memorial Day
   6:2    Italian Founding of the Republic
   6:5    Korean Memorial Day
   6:10   Portugal Day
   6:14   American Flag Day
   6:16   United Nations Soweto Day
   6:17   German Unity Day
   6:29   CIEX Appreciation Day
   7:1    Canada Day
   7:4    American Independence Day
   7:12   Orangemen's Day
   7:14   French Bastille Day
   8:1    Swiss National Day
   8:9    Singapore National Day
   8:16   Pakistan Independence Day
   8:31   Trinidad Independence Day
   9:5    Guy Fawkes Day
   9:16   Mexican Independence Day
   9:17   Citizenship Day
   9:28   Confucius' Birthday
   10:12  Columbus Day
   10:24  United Nations Day
   10:31  Halloween
   11:1   All Saints Day
   11:5   Guy Fawkes Day
   11:5   October Russian Revolution Day
   11:11  American Veterans' Day
   11:20  Mexican Revolution Day
   12:25  Christmas Day
   12:26  Boxing Day

endblock

# # # # # # # # # # # # # # # # # # # # # #

callpoint: interrupt

# Jump here when a CNTRL-C or BREAK key is entered.
# The procedure exits without saving any data.

# # # # # # # # # # # # # # # # # # # # # #

call -goto exit_no_save

# # # # # # # # # # # # # # # # # # # # # #

callpoint: exit

# Leave the program, first listing the current appointment
# information to a file.

# # # # # # # # # # # # # # # # # # # # # #

unset -view
set $appt:appointment_book = $appointment_book
set -view $appt

if -cmd

   list -all > $appointment_book
   scr -def
   return

else

   # Could not save the database.

   scr -buffer off
   scr -def
   scr -clear
   move 1 15
   move -down; echo -scr "ERROR: Cannot create appointments file..."
   move -down; echo -scr "$sys:errtxt $sys:errloc"
   move -down; echo -scr "Is local disk write protected?"
   move -down; echo -scr "Press any key to continue..."
   read -key $confirm
   return -entry  # End the program.

endif

# # # # # # # # # # # # # # # # # # # # # #

callpoint: exception

# When exceptions occur, display error text and return to the
# top. This callpoint is normally executed only if a bug is
# introduced to the script.

# # # # # # # # # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: calendar.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"
move -down; echo -scr "Press any key to continue..."
read -key $arg:exit

return -entry

# # # # # # # # # # # # # # # # # # # # # #

callpoint: exit_no_save

# Leave the program without saving the appointment data.

# # # # # # # # # # # # # # # # # # # # # #

scr -def
return

