# # # # # # # # # # # # # # # # # # # # # # #

# DEMO.S - Demonstration Program Executive
# CIEX (C) 1993, 1994 New Line Software, Inc.
# Script Version 1.1

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Run the CIEX demonstration program.

# # # # # # # # # # # # # # # # # # # # # # #

# Initialize the screen.

set -clear           # Clear all variables from memory
screen -default      # Set screen parameters to default
screen -buffer on    # Turn on screen buffering.
screen -clear        # Clear the screen.
source -calls        # Speed up access to callpoints.

# Draw the root window is as follows:

screen -fore green
box 1 1 80 25

# Heading, at the top of the display, is as follows:

move  1 1 ; echo -center "  THE CIEX INTERPRETER  "
move -down; echo -center "Copyright 1993, 1994 New Line Software, Inc."
move -down; echo -center "Command Interface Executive, Program Demo V2.3"

# Main text box is:

screen -fore red
box -style french
box 5 5 72 14

# Display an introductory message.

screen -fore white
move 10 5; echo -center "  WELCOME TO CIEX!  "
move 10 6

echoblock -scr

This program demonstrates typical applications of the CIEX
interpreter. It executes several different script programs.
Each script consists of a plain ASCII text file in the local
directory, and demonstrates particular capabilities of CIEX.

These demo scripts illustrate only a tiny subset of the many
possible uses of CIEX. You can modify these scripts with
any ASCII text editor (such as the MSDOS "edit" program) or
create completely new scripts. Refer to the README file,
included with the distribution files, for more information.

endblock

# Display the NLS logo:

call display_logo 23 20

# Display a prompt to continue:

scr -fore white
move 1 18; echo -center ' [ ANY KEY TO CONTINUE  ] '

# Flush the screen to the display.
# The initial screen appears here:

screen -flush

# Wait for a key. The program stops until a key entry.

read -key $keyname

# Get the current directory.

set -inline $working_directory = cd

# If the install script exists, run it now.

if install.s is file and ':\CIEX$' not in $working_directory

   # The "install.s" script exists in the current directory, and the
   # current directory is not "\CIEX". Run the installation script.
   # The "inline" option is used by the script to provide an extra menu
   # option, allowing the demo to continue without installing the CIEX
   # software. Note that the "install.s" script may also be executed as
   # a stand-alone script.

   screen -clear
   source install.s inline

endif

# Begin main duty loop for the program.
# Looping continues until the F10 key is entered.

loop

   set -clear         # Remove all variables from memory.
   screen -def        # Set default screen parameters.
   screen -clear      # Clear the screen.
   screen -buf on     # Turn on screen buffering.

   # Display main root window.

   screen -fore green
   box -style roman
   box 1 1 80 25

   # The screen header is as follows:

   move 1 1 ; echo -center " THE CIEX INTERPRETER - DEMO PROGRAM "

   # Invert screen region, for monochrome displays.

   line -invert 80

   # Options box is as follows:

   screen -fore red
   box -style french
   box 8 3 65 19
   screen -fore white

   # Subtitle is:

   move 1 3; echo -center "  ENTER A FUNCTION KEY  "

   # Button labels are as follows. Each button is inverted for
   # a better appearance on monochrome displays.

   screen -fore cyan
   move 13  5   ; echo -scr5 "   F1   "; line -invert 8
   move -down 2 ; echo -scr5 "   F2   "; line -invert 8
   move -down 2 ; echo -scr5 "   F3   "; line -invert 8
   move -down 2 ; echo -scr5 "   F4   "; line -invert 8
   move -down 2 ; echo -scr5 "   F5   "; line -invert 8
   move -down 2 ; echo -scr5 "   F6   "; line -invert 8
   move -down 2 ; echo -scr5 "   F10  "; line -invert 8

   # The button descriptions are:

   scr -fore white
   scr -back black
   move 25 5;    echo -scr "INTRO.S     - An Introduction to CIEX."
   move -down 2; echo -scr "EVAL.S      - A Desk Top Calculator."
   move -down 2; echo -scr "ADDR.S      - An Address Database/Dialer."
   move -down 2; echo -scr "CALENDAR.S  - A Calendar/Appointment Book."
   move -down 2; echo -scr "REACTOR.S   - Home Fusion Reactor Monitor."
   move -down 2; echo -scr "CIEX Product Information and Support."
   move -down 2; echo -scr "EXIT This Program."

   # Display the New Line Software Logo

   move 18 21 ; line -clear
   call display_logo 23 20

   # Loop to get command input:

   loop

      # Display the current screen:

      scr -flush

      # Wait for a key.

      read -key $keyname

      if      $keyname eq f1    # Run the INTRO.S script.

         if intro.s is not file
            call file_not_found
         endif

         call about_intro_s
         source intro.s
         source -calls
         break

      else if $keyname eq f2    # Run the EVAL.S script.

         if eval.s is not file
            call file_not_found
         endif

         call about_eval_s
         source eval.s
         source -calls
         break

      else if $keyname eq f3   # Run the ADDR.S script.

         if addr.s is not file
            call file_not_found
         endif

         call about_addr_s
         source addr.s
         break

      else if $keyname eq f4  # Run the CALENDAR.S script.

         if calendar.s is not file
            call file_not_found
         endif

         call about_calendar_s
         source calendar.s
         source -calls
         break

      else if $keyname eq f5  # Run the REACTOR.S script.

         if reactor.s is not file
            call file_not_found
         endif

         call about_reactor_s
         source reactor.s
         source -calls
         break

      else if $keyname eq f6  # Display information on how to order.

         call ordering_info
         break

      else if $keyname eq f10 # Exit this program.

         break -all

      endif

      # Repeat loop for a user key.

   endloop

endloop

# Display following exit screen.
# Handle any exceptions that are encountered.

if -cmd

   call display_exit_screen

else

   # Exit screen failed. Current disk might be write protected.
   # Display the following, simpler screen.

   scr -def
   screen -clear
   screen -fore red
   box -style roman; box   8 7

   call display_logo 23 11

   screen -fore green
   move  1 9;  echo -center "Thanks for your interest in CIEX!"
   move  1 15; echo -center "Phone: 1-800-441-2931  Fax: 1-303-771-0836"

endif

# Cleanup before exiting.

set -clear
scr -buffer off

# Return to DOS.

return -all

# # # # # # # # # # # # # # #

callpoint: about_intro_s

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 8 2 66 22
screen -fore cyan
move 14 4
echo -center "ABOUT THE INTRO.S SOURCE LISTING..."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The INTRO.S script demonstrates the CIEX interpreter's
ability to handle text and graphics. The program is an
interactive application, which views a document with
numbered paragraphs, and provides user control of the
resulting screens.

The INTRO.S program is a large program. Its source code
may appear slightly cryptic because INTRO.S is generated
by another CIEX script (called VGCOMP) included with the
optional CIEX Development Environment. The VGCOMP script
reads in documents containing numbered paragraphs, and
outputs new CIEX scripts which, when executed, display
screens like the ones that follow.

endblock

screen -buf off
screen -flush

move -home
screen -fore cyan
echo -center '[ ANY KEY TO BEGIN ]'
read -key $input
echo -center '[    LOADING...    ]'

# # # # # # # # # # # # # # #

callpoint: about_eval_s

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 8 2 66 22
screen -fore cyan
move 14 4
echo -center "ABOUT THE EVAL.S SOURCE LISTING..."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The EVAL.S script is a basic desk top calculator. It
is a short program. Its source code is relatively easy
to understand. The largest part of the program is spent
on a lengthy help section, accessed with the F1 function
key. The main work of the program is done by a small
program segment, which gets the user's input and passes
this input to the interpreter for evaluation.

The EVAL.S script shows a number of features of CIEX.
It demonstrates how to use the interpreter's error
handler to trap user input errors. It also shows some
of the interpreter's facilities to organize and scroll
text. Most importantly, the EVAL.S program demonstrates
the interpreter's ability to perform simple or complex
mathematical evaluations.

endblock

scr -buff off
screen -flush

move -home
screen -fore cyan
echo -center '[ ANY KEY TO BEGIN ]'
read -key $input
echo -center '[    LOADING...    ]'

# # # # # # # # # # # # # # #

callpoint: about_addr_s

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 8 2 66 22
screen -fore cyan
move 14 4
echo -center "ABOUT THE ADDR.S SOURCE LISTING..."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The ADDR.S script is a simple database manager and phone
dialer. The program demonstrates many of the CIEX
interpreter's facilities for handling data.

The program holds data in a table that is read from and
stored to the disk file "addr.dat". The operation of the
program is straightforward: Data is input by the user
into the table, and displayed on the screen. To display
the next or previous record, the indices of the table are
changed.

The most complicated functions of the ADDR.S program,
that of sorting data and deleting records, is handled
by the CIEX "renumber" command, which is used to repack
a data array or queue, possibly sorting the data in the
process.

endblock

screen -buff off
screen -flush

move -home
screen -fore cyan
echo -center '[ ANY KEY TO BEGIN ]'
read -key $input
echo -center '[    LOADING...    ]'

# # # # # # # # # # # # # # #

callpoint: about_calendar_s

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 8 2 66 22
screen -fore cyan
move 14 4
echo -center "ABOUT THE CALENDAR.S SOURCE LISTING..."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The CALENDAR.S script is a calendar and appointment
manager. It is similar to the "ADDR.S" program provided
in option F3 of this demo, but provides several twists.
CALENDAR.S is a good demo of a moderately complicated
application requiring data handling, text-graphic, and
nontrivial mathematical evaluation facilities.

The calendar for an arbitrary month is generated with a
widely known algorithm that relies on the start of each
month being a fixed offset from the previous month. The
program as extra logic interleaved throughout to handle
leap years. Today's date is fetched on program startup
using the MSDOS internal "DATE" command.

endblock
screen -buff off
screen -flush

move -home
screen -fore cyan
echo -center '[ ANY KEY TO BEGIN ]'
read -key $input
echo -center '[    LOADING...    ]'

# # # # # # # # # # # # # # #

callpoint: about_reactor_s

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 8 2 66 22
screen -fore cyan
move 14 4
echo -center "ABOUT THE REACTOR.S SOURCE LISTING..."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The REACTOR.S program is a (make-believe) fusion reactor
monitor panel, which provides an overall demonstration
of the CIEX interpreter's text-graphic abilities. The
source code consists mainly of statements needed to draw
the reactor control panel displays.

REACTOR.S demonstrates a special feature of CIEX: the
ability to branch a program on an arbitrary key stroke,
thereby allowing running programs to accept user input.
This is accomplished with the CIEX "trace" facility,
which executes the "trace" callpoint whenever a key is
entered. The REACTOR.S program additionally demonstrates
a major use of the CIEX interpreter: the processing,
formatting, and display of scientific data.

endblock
screen -buff off
screen -flush

move -home
screen -fore cyan
echo -center '[ ANY KEY TO BEGIN ]'
read -key $input
echo -center '[    LOADING...    ]'

# # # # # # # # # # # # # # #

callpoint: ordering_info

# # # # # # # # # # # # # # #

screen -fore red
box -style french
box 5 2 72 22
screen -fore cyan
move 10 3
echo -center "PRODUCT AND ORDERING INFORMATION."
move -down

screen -fore white

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

The base CIEX distribution (which you are now using) includes
the fully functional CIEX interpreter and example programs.
This BASE distribution is FREE!

NEW LINE SOFTWARE ALSO PROVIDES: a 250+ page Command Reference
Manual, a development environment (including debugger, on-line
help, and syntax checker), and script libraries.

OF SPECIAL SIGNIFICANCE TO C-LANGUAGE PROGRAMMERS: New Line
Software offers the CIEX Language Development Call Library
(LDCL), a unique, powerful facility which allows software
developers to create personalized scripting languages directly
from their own object libraries and programs.

To order these options, or request more information, fill-out
the order form enclosed with the CIEX base distribution, or
call NLS at 1-800-441-2931 (or 1-303-741-6594) for help.

endblock

move -home
screen -fore cyan
echo -center '[ ANY KEY TO RETURN TO MENU ]'
screen -flush

read -key $input

# # # # # # # # # # # # # # #

callpoint: file_not_found

# # # # # # # # # # # # # # #

screen -clear

screen -fore red
box -style french
box 8 3 65 21

screen -fore cyan
move 13 5
echo -center "NOTE - CIEX SOURCE FILE(S) NOT FOUND!"
screen -fore white
move -down 1

#23456789 123456789 123456789 123456789 123456789 123456789
echoblock -scr

(!?) You are missing one or more or demonstration files.
Please verify your installation is correct.

In particular, the demo program is written so that all
CIEX script files must be in the current directory and
disk drive.  Try exiting from the demo program, changing
directories so that the demo files reside in the local
directory, and executing the demo program again.

For assistance, please contact New Line Software, Inc.
at 1-800-441-2931, or 1-303-741-6594.

endblock

call display_logo 23 19
move -home
screen -fore cyan
echo -center '[ ANY KEY TO END PROGRAM ]'
scr -flush
scr -buff off

read -key $input

# Return to DOS.

return -all

# # # # # # # # # # # # # # #

callpoint: interrupt
exit

# # # # # # # # # # # # # # #

callpoint: display_logo

# Display the NLS logo.
# Usage: call display_logo (xcoord) (ycoord)

# # # # # # # # # # # # # # #

# Setup variables as follows:

set -view $arg
set $xorigin = $1 # First argument.
set $yorigin = $2 # Second argument.

# Clear the logo region.

box -clear $xorigin  $yorigin 12 3

# Draw block letters.

scr -back black
scr -fore red
box -style roman
box [ $xorigin      ] $yorigin 3 3;
box [ $xorigin + 4  ] $yorigin 3 3;
box [ $xorigin + 8  ] $yorigin 3 3;

# The NLS logo text is as follows:

move [ $xorigin + 1] [ $yorigin + 1 ]; echo -scr "N"
move [ $xorigin + 5] [ $yorigin + 1 ]; echo -scr "L"
move [ $xorigin + 9] [ $yorigin + 1 ]; echo -scr "S"

# Invert each block letter, for monochrome displays.

box -invert [ $xorigin      ] $yorigin 3 3;
box -invert [ $xorigin + 4  ] $yorigin 3 3;
box -invert [ $xorigin + 8  ] $yorigin 3 3;

# Address information.

scr -back black
scr -fore cyan
move [ $xorigin + 14 ] $yorigin
echo -scr "NEW LINE SOFTWARE, Inc."
move -down; echo -scr "P.O. Box 3433"
move -down; echo -scr "Englewood, CO 80155"

# Logo is being displayed. Return.

return

# # # # # # # # # # # # # # #

callpoint: display_exit_screen

# # # # # # # # # # # # # # #

# Clear the screen.

screen -buffer off
screen -clear

# Display CIEX in big letters. The block letters were created using the
# "libfont.s" script (included with the development library) and saved
# to a file using the "screen -save" command. Then, this file was pasted
# below, and is restored to the screen using the "screen -flush" command
# to create the large text display.

# Note: In general, any screen can be saved to a file the CIEX command
# "screen -save (filename)" and subsequently restored using the CIEX
# command "screen -flush (filename)". If (filename) is omitted, the
# screen is saved and restored using the built-in CIEX screen buffer.

echoblock > logo.scr

NLS - New Line Software, Inc.  CIEX - V2.3 (c) 1993, 1994
0720	1019e	0fdb	10006	04db	0720	10003	0fdb
10006	04db	0720	10001	0fdb	1000a	04db	0720
10001	0fdb	10003	04db	0720	10005	0fdb	10003
04db	0720	1001a	0fdb	10002	04db	0720	10001
0fdb	10002	04db	0720	10004	0fdb	10002	04db
0720	10004	0fdb	10002	04db	0720	10004	0fdb
04db	0720	10004	0fdb	10002	04db	0720	10001
0fdb	10002	04db	0720	1001c	0fdb	10002	04db
0720	10003	0fdb	10002	04db	0720	10003	0fdb
10002	04db	0720	10004	0fdb	10002	04db	0720
0fdb	04db	0720	1000a	0fdb	10004	04db	0720
1001e	0fdb	10002	04db	0720	1000b	0fdb	10002
04db	0720	10004	0fdb	10005	04db	0720	1000b
0fdb	10002	04db	0720	1001f	0fdb	10002	04db
0720	1000b	0fdb	10002	04db	0720	10004	0fdb
10002	04db	0720	0fdb	04db	0720	1000a	0fdb
10004	04db	0720	1001e	0fdb	10002	04db	0720
10003	0fdb	10002	04db	0720	10003	0fdb	10002
04db	0720	10004	0fdb	10002	04db	0720	10004
0fdb	04db	0720	10004	0fdb	10002	04db	0720
10001	0fdb	10002	04db	0720	1001d	0fdb	10002
04db	0720	10001	0fdb	10002	04db	0720	10004
0fdb	10002	04db	0720	10004	0fdb	10002	04db
0720	10003	0fdb	10001	04db	0720	10003	0fdb
10002	04db	0720	10003	0fdb	10002	04db	0720
1001d	0fdb	10006	04db	0720	10003	0fdb	10006
04db	0720	10001	0fdb	1000a	04db	0720	10001
0fdb	10003	04db	0720	10005	0fdb	10003	04db
0720	100b9	0320	10001	0354	0348	0341	034e
034b	0353	0320	0346	034f	0352	0320	0359
034f	0355	0352	0320	0349	034e	0354	0345
0352	0345	0353	0354	0320	0349	034e	0320
0354	0348	0345	0320	0343	0349	0345	0358
0320	0349	034e	0354	0345	0352	0350	0352
0345	0354	0345	0352	0321	0320	10001	0720
10016	02d6	02c4	1003a	02b7	0720	10012	02ba
0220	10002	0250	0268	026f	026e	0265	023a
0220	0238	0230	10001	022d	0234	10001	0231
022d	0232	0239	0233	0231	022c	0220	0233
0230	0233	022d	0237	0234	0231	022d	0236
0235	0239	0234	022c	0220	0246	0261	0278
023a	0220	0233	0230	0233	022d	0237	10001
0231	022d	0230	0238	0233	0236	0220	10003
02ba	0720	10012	02ba	0220	1003a	02ba	0720
10012	02ba	0220	1000b	40d5	40cd	40b8	0220
40d5	40cd	40b8	0220	40d5	40cd	40b8	0220
10002	034e	0345	0357	0320	034c	0349	034e
0345	0320	0353	034f	0346	0354	0357	0341
0352	0345	032c	0320	0349	036e	0363	032e
0220	10009	02ba	0720	10012	02ba	0220	1000b
40b3	404e	40b3	0220	40b3	404c	40b3	0220
40b3	4053	40b3	0220	10002	0350	032e	034f
032e	0320	0342	036f	0378	0320	0333	0334
0333	10001	0220	10013	02ba	0720	10012	02ba
0220	1000b	40d4	40cd	40be	0220	40d4	40cd
40be	0220	40d4	40cd	40be	0220	10002	0345
036e	0367	036c	0365	0377	036f	10001	0364
032c	0320	0343	034f	0320	0338	0330	0331
0335	10001	0220	1000d	02ba	0720	10012	02ba
0220	1003a	02ba	0720	10012	02d3	02c4	1003a
02bd	0720	10009

endblock

# Flush the temporary file created above to the screen.

scr -flush logo.scr
screen -fore green
move -home; echo -center '[ ANY KEY TO EXIT ]'

# Remove the temporary file.

erase logo.scr

# Wait for a key.

read -key $a

# Clear the screen before exiting.

screen -default
screen -clear

# Return to DOS.

return -entry

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the demo program.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: demo.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"

exit


