# # # # # # # # # # # # # # # # # # # # # # #

# EVAL.S  - Math Equation Evaluator
# CIEX (C) 1993, 1994 New Line Software, Inc.
# Script version 1.2

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Execute a desk top calculator.

# # # # # # # # # # # # # # # # # # # # # # #

# Jump here when the "clear" key is selected:

callpoint: start

# Initialize variables and the screen.

set -clear
scr -default
source -calls

# These are the colors used with the display.

set $color:text       = white
set $color:title      = cyan
set $color:line       = red
set $color:background = black

# Input parameters are:

set $input:xcoord     = 23
set $input:ycoord     = 22
set $input:equation   = ""
set $input:prompt     = "ENTER EQUATION:"
set $input:cursor     = "_"
set $input:field_size = 54

# Output parameters are:

set $output:xcoord    = 23
set $output:ycoord    = 20
set $output:result    = "+0.000"
set $output:logfile   = eval.log
set $output:fixval    = fix3

set $total = "0.000"

# Setup the background:

scr -buffer on
scr -background $color:background
call setup_screen
scr -buffer off
scr -flush

# Enter main service loop:

loop

   # Display the prompt:

   move 1 $input:ycoord
   scr -fore $color:title
   echo -scr18 $input:prompt
   scr -fore $color:text

   # Get a math statement.

   move $input:xcoord $input:ycoord
   call get_equation

   # A math statement now resides in the value of $input:equation.
   # Clear any existing status line message.

   call update_status_line ""

   # If the input value is null, return to the prompt:

   if -test $input:equation^value is null
      break -next
   endif

   # Add the "fix" operator to the user input, needed to set the number
   # of decimal places in the output. The operant math statement will
   # reside in the variable $temp_eval.

   set $temp_eval = "$output:fixval ("
   set $temp_eval & $input:equation
   set $temp_eval & ")"

   # The main work of the interpreter is done here:

   # Evaluate the equation. If the command is successfully evaluated,
   # display the equation and result in the transcript pad. Otherwise,
   # report an error and do nothing more.

   if -cmd; set -eval $result = $temp_eval

      # The evaluator accepted the equation. The result is in the
      # variable "$result"

   else

      # The evaluator rejected the equation. Display the error and
      # return to the top.

      call update_display "ERROR" $input:equation
      call update_status_line "Entry error: $sys:errtxt"
      break -next

   endif

   # The equation was successfully evaluated.

   if $result^size gt 18

      # The result is too big. Discard.

      call update_display "ERROR" $input:equation
      call update_status_line "!? - Error: Output overflow."
      break -next

   else if $result is not float

      # The user entered some text. The evaluator can process text
      # statements without an error, but we only want to display numeric
      # results. Discard the input.

      call update_display "ERROR" $input:equation
      call update_status_line "!? - Error: Invalid output value."
      break -next

   endif

   # If the prefix is positive, prefix the result by a "+" character.
   # The evaluator normally omits leading "+" characters for positive
   # results.

   if $result ge 0
      set $output:result = "+"
      set $output:result & $result
   else
      set $output:result = $result
   endif

   # Display the equation and result in the transcript pad.

   call update_display $output:result $input:equation

   # If logging is on, append the result to the log.

   if $log_to_file^value is not null

      startblock >> $output:logfile
         echo -fmt18 $output:result
         echo -fmt   ' | '
         echo -value $input:equation
      endblock

   endif

   # Calculate the running total. The value is not used by the program,
   # but is nice to have anyway.

   set $total ? 0.000
   if -cmd
      set -eval $total = $output:fixval ( $total + $output:result)
   endif

endloop

# Program is finished. Exit.

return

# # # # # # # # # # # # # # #

callpoint: get_equation

# Get an equation from the keyboard, terminated with an "Enter" or
# "=" character. A maximum of "$input:field_size" characters are
# fetched. On exit, the equation input by the user resides in the
# variable "$input:equation". Special logic is provided to handle
# function keys, in particular the "prev_result" key.

# # # # # # # # # # # # # # #

# Initial values on entry are:

set $input:equation = ""
set $input:keycount = 0

loop

   # Display the input:cursor and read a key.

   echo -scr $input:cursor
   read -key $input:key

   if -test $input:key eq "enter" or $input:key eq "="

      # The equation has been entered. Break from the loop.
      # Note - this is the only exit point for the loop and callpoint.

      break;

   else if -test $input:key^size eq 1

      # A single key was entered.

      if $input:keycount lt $input:field_size

         # Add the key to the input equation and echo the key.

         set $input:equation & $input:key
         echo -scr "$input:key"
         move -right
         set $input:keycount + 1

      endif

   else if -test $input:key eq "backspace" or $input:key eq "left"

      # Erase the previous character.

      if -test $input:keycount gt 0
         echo -scr " "
         move -left
         set -replace $input:equation = '?$' by ''
         set $input:keycount - 1
      endif

      # See if the key was a function key.

   else if -test $input:key eq f2; call prev_result
   else if -test $input:key eq f5; call -goto start
   else if -test $input:key is callpoint

      call $input:key

   endif

endloop

# Return the value as follows:
# First, stripping any leading blanks.

set -replace $input:equation = '^ +' by ''

# Erase the equation from the display.

move $input:xcoord $input:ycoord
line -clear [ $input:field_size + 1 ]
return

# # # # # # # # # # # # # # #

callpoint: setup_screen

# Initialize the screen, including lines and boxes.

# # # # # # # # # # # # # # #

scr -clear

# The top legend is as follows:

scr -fore $color:title
move 1 1
line -clear
echo -center ~
" F1=Help     F2=Prev Result     F3=Decimal    F4=Log     F5=Clear     F10=Exit"
line -invert

# Initialize the status line at the bottom of the display.

unset $status_text
call update_status_line ""

# Draw the transcript pad.

scr -fore $color:line
box -style single
box  20  21  61   3
move  1   2  ; line -horiz
move  1  21  ; line -horiz
move  1  23  ; line -horiz
move 20   2  ; line -vert 22
move -right  ; line -vert 22

# This section of the screen can be scrolled.

select -scr 1 4 80 17

# Screen is now setup. Return.

return

# # # # # # # # # # # # # # #

callpoint: update_display

# Update the transcript pad display. The first input argument is the
# result to display. The second input argument is the equation to
# display. If the previous result was an error, the transcript pad is
# not scrolled, otherwize the transcript pad is scrolled by one line.

# # # # # # # # # # # # # # #

if $update:last_result^value eq "ERROR"

   # The last result was an error. Clear the last entry.

   scr -fore $color:text
   move 1 $output:ycoord
   line -clear

else # The last result was not an error. Scroll the display.

   scroll -up

endif

# Save this result value:

set $update:last_result = $arg:1

# Add vertical lines to fill gap caused by scrolling

scr -fore $color:line
move 20 20;  line -vertical 1
move 21 20;  line -vertical 1

# Display the result value (i.e. argument # 1)

scr -fore $color:text
move 1 $output:ycoord
echo -scr18 $arg:1

# Display the equation value (i.e. argument #2)

move $output:xcoord $output:ycoord
echo -scr $arg:2

# The equation and result has been added to the transcript pad
# display. Return.

return

# # # # # # # # # # # # # # #

callpoint: update_status_line

# Update the status line at the bottom of the display. First argument
# is the text to be displayed on the status line. The status line is
# changed only when the status text changes.

# # # # # # # # # # # # # # #

set $status_text ? "(null)" # Initialize on first pass.

if $status_text eq $arg:1

   # Status line has not changed.

   return

endif

# Add the text to the bottom of the screen.

set $status_text = $arg:1
move -home
scr -fore cyan
line -clear
echo -scr-55 " $status_text"

# The following is always appended to the status display.

move -right 58; echo -scr-58 "CIEX V2.3, NLS, Inc."
move -home
line -invert

return

# # # # # # # # # # # # # # #

callpoint: prev_result

# Submit the previous result as an operand.

# # # # # # # # # # # # # # #

if -eval $input:keycount + $result^size lt $input:field_size

   # There is room in the input window to submit the previous result.

   echo -scr $output:result
   move -right $output:result^size
   set $input:keycount + $output:result^size
   set $input:equation & $output:result

endif

# # # # # # # # # # # # # # #

callpoint: f3

# Fix the decimal point.

# # # # # # # # # # # # # # #

# Clear any existing input.

move $input:xcoord $input:ycoord
line -clear [ $input:field_size + 1 ]
echo -scr "_"
set $input:equation = ""
set $input:keycount = 0

# Display a prompt:

scr -fore $color:title
move 1 $input:ycoord
echo -scr18 "DECIMAL? (0-9):"

# Get a key:

scr -fore $color:text
read -key $i

# Check the input value:

scr -fore $color:text
if $i llt 0 or $i lgt 9

   # Cannot fix this number of decimal places.

   call update_status_line "!? - Error: Decimal value out of range."

else

   # Update the fix value.

   set $output:fixval = "fix"
   set $output:fixval & $i
   call update_status_line "Decimal fixed at $i places."

endif

# Redisplay the prompt:

scr -fore $color:title
move 1 $input:ycoord
echo -scr18 $input:prompt
scr -fore $color:text
move $input:xcoord $input:ycoord

# Decimal places have been fixed. Return.

return

# # # # # # # # # # # # # # #

callpoint: f4

# Log results to a file.

# # # # # # # # # # # # # # #

# Clear any existing input.

move $input:xcoord $input:ycoord
line -clear [ $input:field_size + 1 ]
echo -scr "_"
set $input:equation = ""
set $input:keycount = 0

scr -fore $color:title

if $log_to_file^value is null

   # Not currently logging to a file. Display the following prompt:

   move 1 $input:ycoord
   echo -scr18 "START LOG? (Y/N):"
   read -key $i

   # Handle the input:

   if $i eq 'Y' or $i eq 'y'

      call update_status_line "Logging to file - $output:logfile ..."
      set $log_to_file = 1

      # Initialize the log file as follows:

      if -cmd

         echo >> $output:logfile
         echo ~
         '-------------------|-------------------------------' ~
         >> $output:logfile

      else

         # Could not create log file.

         scr -buffer off
         scr -def
         scr -clear
         move 1 15
         move -down; echo -scr "ERROR: Cannot create log file..."
         move -down; echo -scr "$sys:errtxt $sys:errloc"
         move -down; echo -scr "Is local disk write protected?"
         move -down; echo -scr "Press any key to continue..."
         read -key $confirm
         call -goto start   # Restart the program from the top.

      endif

   endif

else # Currently logging to a file. Display the following prompt:

   move 1 $input:ycoord
   echo -scr18 "STOP LOG? (Y/N):"
   read -key $i

   # Handle the input:

   if $i eq 'Y' or $i eq 'y'

      call update_status_line "Logging has stopped."
      unset $log_to_file

   endif

endif

# Redisplay the standard prompt.

scr -fore $color:title
move 1 $input:ycoord
echo -scr18 $input:prompt
scr -fore $color:text
move $input:xcoord $input:ycoord

# Logging has been enabled or disabled. Return.

return

# # # # # # # # # # # # # # #

callpoint: f10

# Exit the program.

# # # # # # # # # # # # # # #

scr -back black
scr -buf off
scr -clear
call -goto exit

# # # # # # # # # # # # # # #

callpoint: f1

# Display the help screens.

# # # # # # # # # # # # # # #

# Clear any existing input.

move $input:xcoord $input:ycoord
line -clear [ $input:field_size + 1 ]
echo -scr "_"
set $input:equation = ""
set $input:keycount = 0

# Save the current screen.

scr -save

# Initialize the new screen.

scr -clear
box -style french

# Setup title bar at the top of the screen.

scr -fore $color:title
move 1 1
line -clear
echo -center ~
"(Any Key for More Help ...)                           F10=To Exit Help"
line -invert

# Draw a box around the help text.

scr -fore $color:line
box 1 2 80 23

# First paragraph is as follows:

scr -fore $color:title
move 5 4; echo -scr "HOW TO USE THE EVAL PROGRAM"
scr -fore $color:text
move -down
echoblock -scr

Enter a math equation in the input window. The equation is terminated
with an "Enter" or "=" key. The equation can contain math operators
and selected scientific functions. Evaluations use standard math
precedence (with "*" and "/" operations performed before "+" and "-"
operations). Intermediate quantities can be grouped using (possibly
nested) parentheses.

endblock

# Second paragraph is as follows:

scr -fore $color:title
move 5 13; echo -scr "FUNCTION KEYS"
scr -fore $color:text
move -down

echoblock -scr

[ F1=Help        ]     Display this help message.
[ F2=Prev Result ]     Submit previous results into input window.
[ F3=Decimal     ]     Set the number of decimal places in results.
[ F4=Log to File ]     Append results to the file 'eval.log'
[ F5=Clear       ]     Clear the calculator.
[ F10=Exit       ]     Exit the program.

endblock

# The first screen is being displayed. Wait for a key:

read -key $i

if $i eq f10

   # Return to the main screen:

   scr -flush
   scr -fore $color:text
   move $input:xcoord $input:ycoord
   return

endif

# Continue displaying help as follows:
# Subsequent screens are handled differently as follows:

# Clear the help screen.

box -style clear
box 2 4 78 18

# Display a title for the help screens that follow:

scr -fore $color:title
move 5 4; echo -scr "ALL MATH OPERATORS, IN ORDER OF PRECEDENCE"
scr -fore $color:text
move -down

# Derive the help text. Pages are delineated by a "--NewPage--" keyword.

set -cmd $text = echoblock

int             Integer operator. Return the right floating point number
                rounded to the nearest integer value.

not             Logical negation operator. Return the logical negation
                of the right integer number.

abs             Absolute value operator. Return the absolute value of
                the right floating point number.

floor           Round-down operator. Round the right floating point
                number down to the nearest integer value.

ceil            Round-up operator. Round the right floating point number
                up to the nearest integer value.
--NewPage--
log             Natural logarithm operator. Return the natural (base
                'e') logarithm of the right positive floating point
                number.

exp             Natural exponentiation operator. Return the value of
                'e' raised to the power of the right floating point
                number.

log10           Base 10 logarithm operator. Return the base 10 log of
                the right positive floating point number.

sqrt            Square root operator. Return the square root of the
                right positive floating point number.
--NewPage--
tanh            Hyperbolic tangent operator. Return the hyperbolic
                tangent of the right floating point number.

sinh            Hyperbolic sine operator. Return the hyperbolic sine of
                the right floating point number.

cosh            Hyperbolic cosine operator. Return the hyperbolic cosine
                of the right float point number.
--NewPage--
atan            Arc-tangent operator. Return the arc-tangent of the
                right floating point number. The result is an angle
                expressed in radians.

asin            Arc-sine operator. Return the arc-sine of the right
                floating point number. The result is an angle expressed
                in radians.

acos            Arc-cosine operator. Return the arc-cosine of the right
                floating point number. The result is an angle expressed
                in radians.
--NewPage--
tan             Tangent operator. Return the tangent value of the right
                floating point number. The right value is an angle
                expressed in radians.

sin             Sine operator. Return the sine value of the right
                floating point number. The right value is an angle
                expressed in radians.

cos             Cosine operator. Return the cosine value of the right
                floating point number. The right value is angle
                expressed in radians.
--NewPage--
**              Exponentiation operator. Return the value of the left
                number when raised to the power of the right number.

*               Multiply operator. Multiply the left and right floating
                point numbers.

/               Division operator. Divide the left floating point number
                with the right nonzero floating point number.

+               Addition operator. Return the sum of the left and right
                floating point numbers.

-               Subtraction operator. Return the value of the left
                floating point number when subtracted by the right
--NewPage--
%               Mod operator. Return the remainder of the left floating
                point number when divided by the right floating point
                number.

&               Concatenation operator. Return the catenated result of
                the left and right numbers.

lt              Less than operator. Return 1 if the left floating point
                number is less than the right floating point number.
                Otherwise, return 0.

gt              Greater than operator. Return 1 if the left floating
                point number is greater than the right floating point
                number. Otherwise, return 0.
--NewPage--
le              Less than or equal operator. Return 1 if the left
                floating point number is less than or equal to the right
                floating point number. Otherwise, return 0.

ge              Greater than or equal operator. Return 1 if the left
                floating point number is greater than or equal to the
                right floating point number.  Otherwise, return 0.

eq              Equivalence operator. Return 1 if the left value is
                equal to the right value.

ne              Not equal operator. Return 1 if the left value is not
                equal to the right value.
--NewPage--
nand            Nand operator. Return the result of a logical "not and"
                operation performed on the left and right values.

and             And operator. Return the result of a logical "and"
                operation performed on the left and right values.

nor             Nor operator. Return the result of a logical "not or"
                operation performed on the left and right values.

xor             Xor operator. Return the result of a logical
                "exclusive or" operation performed on the left and right
                values.

--NewPage--
xnor            Xnor operator. Return the result of a logical "not
                exclusive or" operation performed on the left and right
                values.

or              Or operator. Return the result of a logical "or"
                operation performed on the left and right values.

endblock

# Display the text to the screen.

loop for $line = $text

   if -test "^--NewPage--" in $line

      # The page is being displayed. Prompt to continue.

      read -key $i
      if $i eq f10

         # Return to the main screen.

         scr -flush
         scr -fore $color:text
         move $input:xcoord $input:ycoord
         return

      else

         # Clear the screen and prepare the next page.

         box 2 6 78 18
         move 5 6
         break -next

      endif

   endif

   # Display a single line of text, and reposition.

   echo -scr $line; move -down

endloop

# The last page of text is being displayed. Prompt to continue.

scr -fore $color:title
move -down
echo -scr '[ ANY KEY TO RETURN TO CALCULATOR ]'
read -key $i

# Restore the display.

scr -flush
move $input:xcoord $input:ycoord
scr -fore $color:text

# Help facility is finished. Return.

return

# # # # # # # # # # # # # # #

callpoint: interrupt

# Jump here on interrupt.

# # # # # # # # # # # # # # #

call -goto exit

# # # # # # # # # # # # # # #

callpoint: exit

# Exit this script.

# # # # # # # # # # # # # # #

return

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the eval.s program.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: eval.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"
move -down; echo -scr "Press any key to continue..."
read -key $arg:exit

return -entry


