# # # # # # # # # # # # # # # # # # # # # # #

# INTRO.S - Copyright 1993, 1994 NLS Inc. New Line Software
# An Introduction to the CIEX Interpreter.

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Display sections of a text file in nested view graphs.

# # # # # # # # # # # # # # # # # # # # # # #

# Setup the following:

echo -off          # Turn off command echoing.
unset -calls       # Remove any callpoints from the interpreter.
source -calls      # Speed up the access to callpoints in this file.

# Initialize the environment:

unset -view        # Set the root view.
unset -all         # Unset any variables in the root view.
screen  -default   # Begin with default screen parameters.
move    -home      # Move cursor home.

# The $text:content variable contains text for the current screen.
# Initialize to zero.

set $text:content = 0

# The following parameters affect the appearance of the display.

set $boxcolor   = red     # Color of boxes.
set $boxstyle   = roman   # Style of boxes.
set $titlecolor = cyan    # Color of section titles.
set $labelcolor = green   # Ancillary label colors.
set $textcolor  = white   # Color of other text.

# The bullet, used to highlight the default menu select, is:

set -inline $bullet  = echo -ascii 16

# Maximum column is at 75 characters.

set $maxcolumn = 75

# Other parameters used by this script:

set $margin     = $maxcolumn
set $option     = 0  # Key, input by operator.
set $secno      = 1  # First screen to display.

# Instantiate the table of contents.

setblock -cmd $index = call table_of_contents

# Clear the screen.

screen -clear

# Display the initial screen:

set $margin = [$maxcolumn - 2 * $index:$secno:level]
call $index:$secno:sec_name
call draw_screen

# Begin the main service loop.

loop

while  $secno gt 0 ~
   and $secno le $index:max_secno

   # Query for an option. Program stops until a key is input.

   read -key $option

   # Clear the control bar at the bottom of the display.

   line -clear

   # Decode the option, input by the user.

   if      -test $option eq 'f1'  ; call help_screen
   else if -test $option eq 'f3'  ; call prev_screen
   else if -test $option eq 'f5'  ; call home_screen
   else if -test $option eq 'f10' ; call -goto exit_screen

   else if -test $index:$secno:menu_range is not null

      # This screen displays a menu.

      if -test $option is integer

         if $option gt 0 and $option le $index:$secno:menu_range

            # Screen displays a menu, and input is integer selection.
            # Set a new default menu selection.

            set $index:$secno:menu_default = $option

         else

            call flag_default_menu_item
            call draw_ctrl_bar
            break -next         # Go back to the prompt.

         endif

      else if -test $option eq 'down'

         # Screen displays a menu, and input is down arrow.
         # Set default menu selection to the next selection.
         # Then, redraw the box with no other action.

         set $index:$secno:menu_default + 1
         if -test $index:$secno:menu_default gt $index:$secno:menu_range
            set $index:$secno:menu_default = 1
         endif

         call flag_default_menu_item
         call draw_ctrl_bar
         break -next         # Go back to the prompt.

      else if -test $option eq 'up'

         # Screen displays a menu, and input is up arrow.
         # Set default menu selection to the previous selection.
         # Then, redraw the box with no other action.

         set $index:$secno:menu_default - 1
         if  -test $index:$secno:menu_default lt 1
            set $index:$secno:menu_default = $index:$secno:menu_range
         endif

         call flag_default_menu_item
         call draw_ctrl_bar
         break -next         # Go back to the prompt.

      else if $option ne f3  and $option ne f4   ~
         and $option ne enter

         call draw_ctrl_bar
         break -next         # Ignore this key.

      endif

      # Derive the name of the callpoint for the next screen.

      set $new_section = $index:$secno:sec_name
      set $new_section & '.'
      set $new_section & $index:$secno:menu_default

      if -test $index:$secno:menu_default eq $index:$secno:menu_range

         # Go to the previous menu.

         set $index:$secno:menu_default  = 1
         call home_screen
         set $index:$secno:menu_default + 1
         if -test $index:$secno:menu_default gt $index:$secno:menu_range
            set $index:$secno:menu_default = 1
         endif

      else if -test $new_section is callpoint

         # Callpoint is valid. Jump to the specified section.

         call goto_screen

      endif

   else if -test $option eq 'enter' or $option eq 'f4'

      # Go to the next screen.

      call next_screen

   else  # Unknown key. Do nothing.

      call draw_ctrl_bar
      break -next

   endif

   call draw_screen # Draw the current screen.

   # Return to the top of the loop and get the next key entry.

endloop

# Exit the program.

call -goto exit_screen

# # # # # # # # # # # # # #

callpoint: draw_screen

# Draw the screen for this particular level.

# # # # # # # # # # # # # #

# Select the box style, depending upon the screen page number.
# Screens without page numbers have visible box borders.

if -test $text:page eq 0
   box -style $boxstyle
else
   box -style clear
endif

# Draw the box, indenting appropriately, and move to the start of the
# text. The location to place the text is part of the table of contents
# as derived by the VGCOMP script.

screen -foreground $boxcolor

set -eval $origin = $index:$secno:level + 1
set -eval $width  = 80 - $index:$secno:level * 2
set -eval $height = 23 - $index:$secno:level * 2

box  $origin $origin $width $height
move $origin $origin

# Display the box title. The particular style of the box title
# depends upon number of pages in this section.

move -right 2
screen -foreground $titlecolor

if -test $text:page eq 0

   # This screen has no page number.

   if -test $text:title is not null
      move -right 3
      echo -screen " $text:title "
      move -left 3
   endif

else

   # This screen has several pages.

   screen -foreground $labelcolor
   move -down
   move -right 3
   echo -screen $text:title
   move -right 55
   echo -screen "   Page $text:page   "
   move -left  59
   screen -foreground $textcolor

endif

# Display the screen text one line at a time.

screen -buffer on
screen -foreground $textcolor
move -right
move -down 2

if -test $index:$secno:text_range is not null

   set $range = $text:content^range
   loop for $line = 1 to $range
      echo -screen $text:content:$line
      move -down
   endloop

endif

# Draw the menu for this section.
# If no menu exists, the routine returns with no action.

call draw_menu

# Display the control bar at the bottom of the screen.

call draw_ctrl_bar

# The last screen is a special case.
# Display the NLS logo on the last screen.

if $secno eq $index:max_secno
   call display_logo 23 17
   move -home
endif

# The screen is setup. Flush the screen to the display and return.

screen -flush
screen -buffer off
return

# # # # # # # # # # # # # #

callpoint: draw_menu

# If a menu exists on this screen, display the menu.

# # # # # # # # # # # # # #

set -eval $xcoord = $index:$secno:level + 2
set -eval $ycoord = $index:$secno:text_range + $index:$secno:level + 4
set $range = $text:menu^range

move $xcoord $ycoord

if -test $range is null

   # A menu does not exist exists.

   return

endif

screen -foreground $boxcolor
box $xcoord $ycoord [$margin + 3] [$range * 2 + 3 ]
move -right 1

screen -foreground $titlecolor
move -right 2
echo -screen " TOPICS IN THIS SECTION "
move -rel 6 2

# Draw the menu one line at a time.

loop for $line = 1 to $range

   # Draw this menu label.

   screen -foreground $labelcolor
   set $fkey = $line
   echo -screen "   $fkey   "
   line -invert 7

   # Update the menu selection title.

   move -right 12
   screen -foreground $textcolor
   echo -screen "$text:menu:$line"
   move -left  12
   move -down 2

endloop

# Flag the default menu item:

call flag_default_menu_item

move -home
screen -background black

# The menu of topics for this section (if any) is setup. Return.

return

# # # # # # # # # # # # # #

callpoint: flag_default_menu_item

# Update the bullet next to the default menu selection.

# # # # # # # # # # # # # #

screen -foreground $titlecolor
set $xcoord = [ $index:$secno:level + 9]
set $ycoord = [ $index:$secno:text_range + $index:$secno:level + 6]
set $range = $text:menu^range

move $xcoord $ycoord

# Mark the current menu selection.

if -test $index:$secno:menu_default gt $index:$secno:menu_range

   # Adjust the menu default as needed.

   set $index:$secno:menu_default = 1

endif

set $current_menu_item = $index:$secno:menu_default

loop for $line = 1 to $range
   if -test $line eq $current_menu_item
      echo -screen $bullet
   else
      echo -screen "  "     # Remove any existing bullet.
   endif
   move -down 2
endloop

# The default menu item is highlighted. Return.

return

# # # # # # # # # # # # # #

callpoint: draw_ctrl_bar

# Draw the control bar at the bottom of the screen.

# # # # # # # # # # # # # #

screen -background black
screen -foreground $titlecolor
move -home
echo -center ~
'F1=Help     F3=Prev Screen     F4=Next Screen     F5=Prev Menu     F10=Exit'
move -home

# The menu bar at the bottom of the screen is setup. Return.

return

# # # # # # # # # # # # # #

callpoint: help_screen

# Pop the help menu onto the screen.

# # # # # # # # # # # # # #

set $text:title = " NAVIGATING THIS PROGRAM"
set $index:$secno:level + 1
set $text:page  = 0
set $text:menu  = 0

set -cmd $text:content = echoblock
 Selecting the "F4" or "Enter" key will display the next screen in
 sequence.  Selecting "F3" will display the previous screen. The
 "F5" key will display the previous menu. Topics can be selected
 from a menu using the "up" and "down" arrow keys, or by entering
 a topic number. The next menu selection in sequence is identified
 with a ">>" bullet.

 [  F1            ]   - Display this help screen.
 [  F3            ]   - Display the previous screen.
 [  F4 or Enter   ]   - Display the next screen.
 [  F5            ]   - Return to the previous topic menu.
 [  F10           ]   - Immediately end the program.
endblock

call draw_screen

# The help menu is being displayed. Wait for a user input.

read -key $a

# User entered a key. Reset the section name to its original value.
# The calling program will redisplay the previous screen.

set $index:$secno:level - 1
call $index:$secno:sec_name

return

# # # # # # # # # # # # # #

callpoint: next_screen

# Draw the next screen in the sequence.

# # # # # # # # # # # # # #

set $prev_level = $index:$secno:level
set $secno + 1

if -test $index:$secno^value is null

   set $secno = 1
   loop for $i = 1 to $index:max_secno
      set $index:$i:menu_default = 1
   endloop

else if -test $index:$secno:level lt $prev_level

   # This is the end of this section. Display the previous menu.

   set $secno - 1
   call home_screen

   # Automatically bump default menu selection by one.

   set $index:$secno:menu_default + 1
   if  -test $index:$secno:menu_default gt $index:$secno:menu_range
      set $index:$secno:menu_default = 1
   endif

endif

# Bypass any blank screens:

loop while $index:$secno:text_range is null

   call $index:$secno:sec_name
   call draw_screen
   set $secno + 1

endloop

set $margin = [$maxcolumn - 2 * $index:$secno:level]
call $index:$secno:sec_name

# Data for the next screen is setup. The calling program will
# display this screen. Return.

return

# # # # # # # # # # # # # #

callpoint: prev_screen

# Draw the previous screen in the  sequence.

# # # # # # # # # # # # # #

set $this_level = $index:$secno:level
set $this_sec_name = $index:$secno:sec_name

if -test $secno eq 1 # No previous screen exists.
   call -goto exit_screen
endif

set $secno - 1

# Go to the home menu if level is different from before,
# and if the new screen is not a menu.

if -test $index:$secno:sec_name not in $this_sec_name ~
   and $index:$secno:level ne $this_level

   # Previous screen is at different level. Go to home screen.

   set $secno + 1
   call home_screen
   return

else if -test $index:$secno:text_range is null

   # Previous screen is blank. Go to home screen.

   set $secno + 1
   call home_screen
   return

endif

set $margin = [$maxcolumn - 2 * $index:$secno:level]
call $index:$secno:sec_name

# Data for the previous screen is setup. The calling program will
# display this screen. Return.

return

# # # # # # # # # # # # # #

callpoint: home_screen

# Go to the previous screen with a menu.
# (This is the home screen for this subsection.)

# # # # # # # # # # # # # #

if -test $secno eq 1
   call -goto exit_screen
endif

# Scan for index number of home section:

set $this_sec_name = $index:$secno:sec_name
set $secno - 1
loop

   if -test $index:$secno:menu_range is not null ~
      and $index:$secno:sec_name in $this_sec_name

      break

   else if -test $secno eq 1

      break

   endif

   set $secno - 1

endloop

set $margin = [$maxcolumn - 2 * $index:$secno:level]
call $index:$secno:sec_name

# Data for the home screen is setup. The calling program will
# display this screen. Return.

return

# # # # # # # # # # # # # #

callpoint: goto_screen

# Go to a particular screen.

# # # # # # # # # # # # # #

loop while $new_section lgt $index:$secno:sec_name
   set $secno + 1
endloop

# Display any blank screens immediately.

loop while $index:$secno:text_range is null

   call $index:$secno:sec_name
   call draw_screen
   set $secno + 1

endloop

set $margin = [$maxcolumn - 2 * $index:$secno:level]
call $index:$secno:sec_name

# Data for the specified screen is setup. The calling program will
# display this screen. Return.

return

# # # # # # # # # # # # # # #

callpoint: display_logo

# Display the NLS logo.
# Usage: call display_logo (xcoord) (ycoord)

# # # # # # # # # # # # # # #

# Setup variables as follows:

set -view $arg
set $xorigin = $1 # First argument.
set $yorigin = $2 # Second argument.

# Clear the logo region.

box -clear $xorigin  $yorigin 13 4

# Draw block letters.

screen -back black
screen -fore red
box -style roman
box [ $xorigin      ] $yorigin 3 3;
box [ $xorigin + 4  ] $yorigin 3 3;
box [ $xorigin + 8  ] $yorigin 3 3;

# The NLS logo text is as follows:

move [ $xorigin + 1] [ $yorigin + 1 ]; echo -scr "N"
move [ $xorigin + 5] [ $yorigin + 1 ]; echo -scr "L"
move [ $xorigin + 9] [ $yorigin + 1 ]; echo -scr "S"

# Invert each block letter, for monochrome displays.

box -invert [ $xorigin      ] $yorigin 3 3;
box -invert [ $xorigin + 4  ] $yorigin 3 3;
box -invert [ $xorigin + 8  ] $yorigin 3 3;

# Address information.

screen -back black
screen -fore cyan
move [ $xorigin + 14 ] $yorigin
echo -screen "NEW LINE SOFTWARE, Inc."
move -down; echo -screen "P.O. Box 3433"
move -down; echo -screen "Englewood, CO 80155"

# Logo is being displayed. Return.

return

# # # # # # # # # # # # # #

callpoint: exit_screen

# Exit the program.

# # # # # # # # # # # # # #

unset -calls      # Remove any callpoints.
screen -default   # Reset screen parameters.
move -home        # Move the cursor home.
return -entry     # End the program.

# # # # # # # # # # # # # #

callpoint: list_source

# List the source document to the screen, or to a file.

# This callpoint is not used by the viewing program, but is provided
# as a convenience to list the source program to standard output, such
# as for possible redirection to a printer.

# At the CIEX prompt, source the callpoints in the viewing program
# with the command: source -calls (thisprogramname)

# Then, from the keyboard, issue the command: call list_source

# # # # # # # # # # # # # #

set -clear
source -calls
set $margin = 70   # The output margin. This can be respecified.

# Instantiate the table of contents.

setblock -cmd $section_list =  call table_of_contents

# Loop for each section number

loop for $i = 1 to $section_list^range

   if $section_list:$i:sec_name^value is null
      break -next  # Skip missing sections of the document
   endif

   # Derive the paragraph number for this section.

   set $paragraph_number = $section_list:$i:sec_name
   set -rep $paragraph_number = '^A.+'

   # Skip a line and display the paragraph number.

   echo ""
   if $paragraph_number^value is not null
      echo -fmt-10 $paragraph_number
   endif

   # Append the section title to the output line.

   echo $section_list:$i:title

   # Get the text for this section.

   call $section_list:$i:sec_name

   if $text:content^range gt 0

      # Text exists for this section.
      # Echo the text to output one line at a time. Indentation can be
      # changed.

      echo ""
      loop for $line = $text:content
         echo {   $line}  # Note - Brackets preserve double quote marks.
      endloop

   endif

endloop

# The source document was listed.
# Append copyright notice and address.

echoblock

CIEX - Copyright 1993, 1994. All rights reserved.
NEW LINE SOFTWARE, INC.
P.O. Box 3433, Englewood, CO 80155
Phone: 1-800-441-2931 or 1-303-741-6594
Fax: 1-303-771-0836

endblock

return -entry

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# File: INTRO.S - VGCOMP data file.
# Constructed from input file:  INTRO.TXT
# This program contains 58 different topics.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A

set $text:title = AN INTRODUCTION TO THE CIEX INTERPRETER
set $text:page = 0

set -cmd $text:menu  = echoblock   # # # Start menu
What Is CIEX?
Why CIEX?
How Is CIEX Used?
The Best Thing About CIEX ...
Return to Prompt
endblock                           # # # End menu

set -cmd $text:content = echoblock -wrap $margin

        Welcome! This program provides an introduction to CIEX: what it
        is, how it is unique, and how it may be used. To navigate this
        program, select "Enter" to proceed to each screen in sequence,
        or use a function key as indicated by the bottom legend.
        Topical screens can be selected from menus by using the "up" and
        "down" arrow keys, or by entering a topic number. Let's begin....

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1

set $text:title = WHAT IS CIEX?
set $text:page = 0

set -cmd $text:menu  = echoblock   # # # Start menu
A Command Interpreter
A Different Way Of Programming
A Different Way Of Thinking
Previous Topic List
endblock                           # # # End menu

set -cmd $text:content = echoblock -wrap $margin

        CIEX can be viewed in several distinct ways. Foremost, CIEX is
        a very high level language (VHLL) with a flexible and powerful
        syntax.  CIEX is also the name of the executable program which
        reads in commands from the keyboard or from a file and executes
        these commands. Finally, CIEX is a programming philosophy which
        can be used to quickly create highly structured, and powerful
        application programs.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.1

set $text:title = A COMMAND INTERPRETER
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX (pronounced "SY - ex") is an acronym for "Command Interface
        Executive". As the name implies, the CIEX interpreter is built
        around a command line prompt which accepts, interprets, and
        executes user directives. At this level of operation, CIEX is
        very simple to use. CIEX syntax is based upon a small set of
        commands which are expanded upon by a multitude of options. CIEX
        commands allow data to be stored and listed, subprograms to be
        executed, special functions to be performed, and mathematical
        expressions to be evaluated. At its root, CIEX is a human
        interface to the computer in the same class as the MS-DOS
        "command.com" program or Unix shell programs. But, depending
        upon the sophistication of the user, CIEX can be vastly more
        powerful than other command line interpreters.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.1.1

set $text:title = INTRODUCTION TO CIEX SYNTAX
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX is a simple but powerful general purpose programming
        language. Its syntax is a blend of DOS batch programs, Unix
        shell scripts, and high level languages such as C, Fortran, Ada,
        and others. It is a language which is adept at processing data
        and text. It includes mathematical expression evaluation,
        nonvolatile storage of data, pattern matching, and text-graphic
        mode functions. CIEX has a free-field, token-based format which
        allows continuation lines. It is a highly structured programming
        language, incorporating block and call facilities, block
        redirects to files, and object-oriented aspects which encourage
        the development of call libraries.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.1.2

set $text:title = STRUCTURED PROGRAMMING
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX programming language includes "if", "else if", "endif"
        branching constructs, "loop" and "endloop" looping constructs
        with "for", "while", "break", and "break -next" modifiers.
        Additionally, CIEX allows subprograms to be defined and nested.
        Arguments may be passed to subroutines by value. Program blocks
        can be defined, and the output of these blocks redirected to a
        file, or automatically assigned to a variable array. Output
        redirection to the disk or to a piped command may be deeply
        nested. In addition to structured flow of control, CIEX allows
        highly structured data as well!

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.1.3

set $text:title = ADVANCED FEATURES
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX incorporates features more powerful than those found in
        Unix shell scripts or DOS batch programs. In particular, CIEX
        allows multidimensional arrays and definition of data records,
        can evaluate complex mathematical statements, and provides
        unique ways to access, filter and control data. CIEX allows
        subroutines to exist in separate files which can be included
        into the main program, thereby setting up run-time libraries
        of external calls. CIEX additionally incorporates both buffered
        and unbuffered text graphic functions to allow boxes to be
        created and positioned, lines to be drawn, text colors to be
        changed, and screen attributes to be set. Also, among the other
        features of CIEX, is the ability to create an ".EXE" file from
        a CIEX script, to promote data privacy and security.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.1.4

set $text:title = EXTENSIBLE INTERFACE
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        In addition to acting as a general-purpose programming language,
        CIEX has a unique architecture which is designed to support
        extensibility. Via the "CIEX Language Development Call Library"
        (CIEX LDCL), C-language (and other) programmers can use CIEX
        as a generic command line interface for programs and object
        libraries, providing scripting capabilities otherwise
        impossible to achieve. CIEX is designed with many elements to
        support easy extensibility, and is therefore at home in
        process control applications, automatic test, and as an API
        for object-library designers. Used in along with the other
        advanced features of CIEX, large specialized systems can be
        developed completely with scripts, greatly enhancing system
        maintainability and ease of development.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.2

set $text:title = A DIFFERENT WAY OF PROGRAMMING
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX interpreter and its associated programming language
        yield a much simpler way of developing software programs than
        compiled languages. Applications which don't require great
        execution speed can be developed much more easily using an
        interpretive language. The development process in such a case
        is much more interactive: The user types commands into a
        keyboard to experiment with a command (if needed), then adds
        the commands to an executable script file. The state of the
        data and program are immediately obtainable from the keyboard
        upon program interrupt or error.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.2.1

set $text:title = BRIEF DATA TYPING
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        If one works with a compiled language for a long period of time,
        it is easy to forget that data typing is not so much a
        consequence of structured design, but a limitation imposed
        upon programmers by compilers (which need to know the size of
        data objects so that memory can be allocated.) The confusion
        can be blamed on equating "data typing" with "data abstraction",
        that is, equating the act of forcing a particular type on a
        storage location with the act of structuring data in a
        particular format. The fact that CIEX is not a strongly typed
        language (but can still define record structures) is a major
        reason why powerful CIEX programs are so easily developed.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.2.2

set $text:title = DATA ABSTRACTION
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX record structures and arrays can be defined and copied
        without any need for a formal declaration. What's more, record
        structures and arrays can be arbitrarily modified during
        execution to add new elements and structure members. In lieu of
        a formal declaration, data objects may be listed from the
        keyboard to determine their structure. CIEX data structures may
        be passed to subroutines by value, listed to external files, and
        re-input to the interpreter by the same or different program.
        The distinction between CIEX's data abstraction and that
        provided by other languages is the completely dynamic way in
        which structures can come into being, and can be modified at
        will.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.2.3

set $text:title = DATA REDIRECTION
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The way CIEX moves data is an outgrowth of the stream functions
        found in MS-DOS and Unix systems. First, CIEX allows easy
        redirection of data from disk files into arrays and back into
        files. Second (and more importantly), CIEX allows redirection
        of data from the output of commands directly into variables. For
        example, to get a directory listing from the disk into an array
        takes exactly one command. While not immediately obvious, the
        ability to move data is one of the most important functions a
        programming language can offer. CIEX provides mechanisms to do
        this smoothly and with little fuss.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.2.4

set $text:title = HIGH LEVEL FUNCTIONS
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The goal of all programming languages should be to take the
        programmer to as high a level as possible. The advantages of a
        computer language over keying binary numbers into memory is that
        the programmer can think in loftier terms. CIEX works at a very
        high level. Simple commands can pack a large punch. This is
        because of the large number of options associated with each
        CIEX command; a command option is usually available to achieve
        the precise result desired. Or, if several commands are needed
        to achieve a result, the programmer can easily write a "call"
        to simplify and structure the program, or use a predefined call
        available in an external library. Through such facilities as
        nested output redirection, nested calls, nested data structures,
        CIEX constantly strives to promote the programmer to higher
        levels.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.3

set $text:title = A DIFFERENT WAY OF THINKING
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        Any good programming language fosters a particular way of
        thinking. Programmers who are familiar with Unix scripting
        languages will attest to the totally different way of
        programming in C-language and C-shell scripts. One thinks
        about a problem and its solution in dramatically different ways.
        The interpretive nature of the language is certainly part of it.
        The friendly interaction with the interpreter is much more
        conducive to experimentation and creativity. But in a broader
        sense, the CIEX interpreter is a telescope on a problem,
        while a compiled language is a microscope. The CIEX
        interpreter selects from the many possible attributes
        associated with large macro functions; compiled programming
        languages solve problems in terms of molecular constituents.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.3.1

set $text:title = THE CIEX PHILOSOPHY OF INTERPRETERS
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        To the user of a program, there is not much visible difference
        between interpreted and compiled languages. Depending upon the
        application, a compiled language may (or may not) run faster
        than an interpreted language. Either way, the user perceives
        no major difference. But to the programmer, the differences
        can be dramatic. An interpreter acts as a go-between for the
        programmer and the function he or she is programming. A compiler
        isolates the programmer from the program's function. Consider
        the differences between reading an English translation of a
        foreign book or talking to the foreign national using a human
        translator. The former is like a program compiler, and is
        understandably faster. The latter is like a program
        interpreter, and can convey nuances not otherwise achievable.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.1.3.2

set $text:title = CIEX IN THE ABSTRACT
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX interpreter has deep roots in Unix, however it is not
        specifically a Unix application. CIEX was developed to amplify
        and enhance the positive aspects of Unix scripting languages,
        and combine these features in DOS (and other) environments. In
        its most abstract form, CIEX may be considered to be a
        programming methodology rather than a program syntax. It
        provides fundamental tools with which to develop new types of
        algorithms, new ways of programming, and new ways of thinking
        about programming.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2

set $text:title = WHY CIEX?
set $text:page = 0

set -cmd $text:menu  = echoblock   # # # Start menu
CIEX For Small Scale Development
CIEX For Medium Scale Development
CIEX For Large Scale Development
Previous Topic List
endblock                           # # # End menu

set -cmd $text:content = echoblock -wrap $margin

        There are many things about the CIEX interpreter which make it
        a unique programming language. CIEX has strong superficial
        similarities to DOS batch files, Unix shell scripts, even such
        compiled languages as C, Fortran, and Ada. The more interesting
        aspects of CIEX are not its similarities to existing languages,
        but its differences, as demonstrated by its application in a
        variety of situations.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.1

set $text:title = CIEX FOR SMALL SCALE DEVELOPMENT
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX is a concise language which imposes very little overhead
        on the programmer. As such, it is well suited for brief
        batch-type jobs such as maintaining files, scanning, or
        filtering data. While CIEX contains specific elements to
        create highly structured and modular programs, it also contains
        facilities such as a "goto", which allow small scale programs to
        be rapidly developed and executed. More esoteric command options
        (which might not be useful for a small job) are nested out of
        sight. Although CIEX is a full-featured language, many useful
        CIEX applications can be created using just a small subset of
        CIEX commands.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.1.1

set $text:title = CIEX AS A SCRIPTING LANGUAGE
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        In the tradition of Unix shell programs, brief CIEX programs
        are often referred to as "scripts", since they tell the
        interpreter the precise functions to execute just as a script
        tells an actor what to do. Commands that a user might type at
        the keyboard, including DOS commands, can be entered into a file
        and passed through the interpreter to cause the same result.
        CIEX can be used anywhere that a DOS batch file is used, often
        with more speed, more flexibility, better graphic displays,
        and better error handling.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.1.2

set $text:title = SIMILARITIES TO DOS BATCH FILES
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        In general, CIEX programs are eminently readable by
        someone who has good knowledge of DOS Batch file programming.
        The CIEX programming language includes functions familiar to
        batch programmers, such as a "goto" capability, "for" loops,
        pipes ("|") and redirects (">" or ">>") to files. CIEX
        programs can directly access DOS commands and execute other
        programs residing on the disk, thereby acting as a command
        interpreter similar to the "command.com" program. Many of the
        CIEX commands (such as "echo" and "set") are synonymous with
        MS-DOS commands.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.1.3

set $text:title = SIMILARITIES TO UNIX SHELL SCRIPTS
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        In many ways, CIEX programs are much closer to Unix shell
        scripts than to MS-DOS batch files. Like Unix shell scripts,
        CIEX programs are case-sensitive, i.e. upper case and lower
        case letters are interpreted differently. Like Unix shell
        scripts, variable identifiers are distinguished from other
        identifiers by a leading '$' character. Like Unix shell scripts,
        the CIEX interpreter supports and relies on inline
        execution of commands and redirection of command output to
        variables and loop control structures.  Unix shell programmers
        can easily adapt existing shell algorithms to CIEX scripts. For
        programmers familiar with the Unix "awk" utility, CIEX easily
        implements the same functionality, often in a more logical and
        readable form.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.1.4

set $text:title = DIFFERENCES FROM OTHER LANGUAGES
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX incorporates many features more powerful than those
        found in Unix shell scripts or DOS batch programs. In addition
        to allowing multidimensional arrays, data records, multipass
        evaluation of mathematical statements, and calls to routines
        in external files, CIEX provides a consistent way of
        handling errors and interrupts, including the ability to trap
        interrupts and exceptions. CIEX was designed for consistency
        and predictability. While a multitude of options can exist for a
        given command, various commands which perform radically diverse
        functions share common options, allowing the programmer to
        intuit their meaning. The result is a highly integrated, user
        friendly, logical language which programmers can use right away
        for simple jobs.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.2

set $text:title = CIEX FOR MEDIUM SCALE DEVELOPMENT
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        Traditionally, batch and script programs can be used for
        medium-scale jobs, but only by taxing their full limit. CIEX
        is very comfortable with medium-sized jobs, such as translation
        of large data systems, simple graphical user interfaces, and
        flat file data bases. CIEX incorporates various features
        to assist in the development and debugging of medium-sized
        systems.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.2.1

set $text:title = DATABASE APPLICATION PROGRAMMING
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The way that CIEX implements data structures makes it ideal
        for programming custom databases. Much of CIEX is similar to a
        database manager, incorporating functions to store data
        structures to disk files, fetch attributes of variables, and
        index records with "first", "next", "prev", and "last"
        modifiers. CIEX incorporates facilities to sort structures by
        members. It allows portions of a database to be made
        inaccessible to selected program segments. A CIEX program can
        determine the current type of data stored in a variable
        location, and modify the data to a new type as needed. These
        functions, combined with the various ways in which CIEX can scan
        arrays (possibly using wildcards to find value matches), make it
        a worthwhile programming language for developing medium-sized
        database applications.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.2.2

set $text:title = DATA CONVERSION PROGRAMMING
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX is excellent as a parsing language. It can break input
        data into tokens of arbitrary length and type, convert tokens to
        new values, seek forward and backward in array structures, and
        match tokens using wildcards. It can also perform substitution
        using wildcards. For clear-text data processing (including
        global editing of text files) CIEX achieves results similar
        to Unix "awk" and "sed" utilities, but in a much more logical
        and easy-to-understand way. The ability to create intermediate
        variables without need for formal declaration allows creation
        of expanding stacks. Flow-of-control facilities provide all the
        tools to make state machines and match grammar rules.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.2.3

set $text:title = MATHEMATICAL AND SCIENTIFIC PROGRAMMING
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX is very useful for scientific programming. The language
        includes support for scientific functions including
        trigonometric functions, exponentiation, and rounding functions.
        CIEX also supports mixing logical operations (including
        "and", "or", "nand", "nor", "xor" and "not") with mathematical
        operations. CIEX performs math operations in floating point
        arithmetic. Data can be formatted to a variety of precision.
        Less obvious (but perhaps more important) is the way that CIEX
        implements data structures and multidimensional arrays. This
        makes the language perfect for matrix and table arithmetic. One
        of the original precepts of CIEX was to provide data analysis
        and reduction. It has deep roots in supporting math intensive
        programming.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.2.4

set $text:title = OFFICE AUTOMATION
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        A major application of interpretive and batch processing
        languages is in the automation of tasks. DOS Batch files have
        become deeply entrenched in the workplace as a means of
        automating activities such as moving and rearranging data files,
        providing menus, and assisting (possibly) computer-naive
        workers. CIEX has major applications in this area. In addition
        to providing all the features normally associated with DOS batch
        files, CIEX allows the easy construction of attractive menu
        displays, including support for single key entries and multiple
        screen colors. CIEX exception handling can result in
        bulletproof interfaces without laborious operator input
        checking. Combined with other features discussed in this
        section, CIEX is a powerful tool for automating an office.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.3

set $text:title = CIEX FOR LARGE SCALE DEVELOPMENT
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        DOS and Unix scripting languages are often abandoned as large
        scale development tools, and rightly so. Beyond a certain point,
        the size of the project overwhelms most interpreters. A
        programmer is forced to use more traditional system programming
        languages. CIEX can easily support large scale development of
        programs with many thousands of lines of code. CIEX includes
        facilities to help develop, debug, organize, and manage large
        scale CIEX projects.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.3.1

set $text:title = SYSTEM PROGRAMMING
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The hallmark of a system program is that it is developed in
        many distinct components, possibly by different designers.
        These components interact to form a highly complicated programs
        with great functionality and possibility of error.
        Object-oriented programming has come into being largely in
        response to the special difficulties of managing the development
        of software systems containing diverse components. CIEX, as an
        object-oriented language, supports the concept of objects,
        classes, call libraries, and data encapsulation. It also
        contains specific elements needed to handle exceptions, and list
        error information.  Because CIEX variables persist on program
        exit or interrupt (unless specifically cleared), the precise
        state of the system on fault or interrupt is immediately
        discernible.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.3.2

set $text:title = SYSTEM CONFIGURATION MANAGEMENT
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        A perceived drawback to non-compiled languages is that they are
        difficult to manage. Because the source code and the executable
        code are the same (as distinguished from compiled language), it
        is thought that non-compiled programs are particularly
        susceptible to spurious and bug-inducing changes. CIEX
        provides a mechanism to bind source code to the interpreter to
        create a single ".EXE" file which resists change and casual
        inspection by the user. This is a useful security measure when
        working with systems which may have many designers or revisions
        in code, since it affords a simple mechanism to prevent
        unwarranted changes to the CIEX source code.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.3.3

set $text:title = LARGE SCALE AUTOMATION
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        Possibly the greatest qualification of CIEX as a large scale
        programming tool is its capacity. CIEX scripts, which reside in
        disk files, are limited in size only to the available disk
        space. Depending upon the amount of memory configured in a
        system, CIEX can hold 1500-plus common variables in memory
        and can easily overlay an unlimited number of new variables from
        disk files. CIEX additionally can hold the location of 500
        library calls in memory, replacing these call locations as
        needed. Various caches are maintained to insure the rapid
        access of any variable or external call. Very large scale
        automation is not only achievable, but a specific intent of the
        CIEX interpreter.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.2.3.4

set $text:title = EXTENSIBILITY
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX supports the design of systems in one specific and
        important way: CIEX is extensible. In fact, one of the original
        purposes of CIEX was to provide generic command line
        interfaces which could be added to any existing program or
        call-library. CIEX is an acronym for "Command Interface
        Executive", implying more than merely a programming
        language. Using the optional CIEX Language Development Call
        Library, C-language (and other) programmers can develop their
        own interpreters from CIEX which satisfy precise needs in
        products and systems. Using CIEX as the "glue", entire systems
        may be scripted, achieving extraordinary flexibility,
        maintainability, and cost-reduction.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3

set $text:title = HOW IS CIEX USED?
set $text:page = 0

set -cmd $text:menu  = echoblock   # # # Start menu
Getting Started With CIEX
Basic CIEX Syntax
Advanced CIEX Syntax
Previous Topic List
endblock                           # # # End menu

set -cmd $text:content = echoblock -wrap $margin

        CIEX is easy to use. It is a programming language which has
        taken great care in simplifying program development,
        maintainability, and understanding. It requires very little
        setup to get started. With the aid of the CIEX Command
        Reference Manual, experienced programmers can immediately become
        productive with the language.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.1

set $text:title = GETTING STARTED WITH CIEX
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX will not run on a write protected disk. Best results
        can be achieved when running on a hard disk of a PC 386 or
        486 computer.  CIEX will run on earlier PCs, and on floppy
        disks, but the performance of most CIEX applications will be
        degraded. The CIEX interpreter consists of a single
        executable file called "CIEX.EXE". To run this program, the
        operator simply types "ciex" at a command line prompt.
        (Alternate ways of starting the CIEX interpreter are also
        possible, as described in the Command Reference Manual.)
        To leave CIEX and return to the DOS prompt, the user types
        "exit". While in CIEX, scripts can be executed using the 
        "source" command. The CIEX prompt also accepts DOS internal
        and external commands just like the DOS prompt.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.1.1

set $text:title = CREATING A CIEX PROGRAM
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX programs are typed into plain text files using the MS-DOS
        "edit" text editor, or some other editor of the programmer's
        choice. The programmer can, of course, invoke an editor at the
        CIEX prompt just like any other external DOS command. CIEX
        does not come with a particular editor; this is in keeping with
        the philosophy that programmers are most productive with their
        own familiar editors. The format of the CIEX program is
        flexible: all tokens in the file must be separated by at least
        one space. CIEX is otherwise space insensitive. In the
        tradition of more modern programming languages, CIEX is case
        sensitive, i.e. upper and lower case letters are treated
        differently. Comments can appear anywhere in the program and are
        regarded as any characters following a "#" character.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.1.2

set $text:title = EXECUTING A CIEX PROGRAM
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX programs can be started from the CIEX command line, or
        from the DOS prompt. The CIEX program accepts a single source
        file argument when it is invoked. If launched with an argument,
        CIEX bypasses interactive operation and proceeds immediately to
        execute the specified file, returning to the DOS prompt when
        finished. If invoked at the CIEX command line (using a
        "source" or "call" command), the interpreter does not exit when
        the source file is finished, and the variables set by the CIEX
        program persist, thereby allowing in depth inspection, such as
        with the CIEX "list" command.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.2

set $text:title = BASIC CIEX SYNTAX
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX command set is small, but flexible. A complete
        description of the CIEX programming language is available
        in the optional CIEX Command Reference Manual, available
        from New Line Software, Inc. A more succinct (but still
        comprehensive) list of CIEX commands is provided in the
        README file which accompanies this base distribution.
        Together with the example programs, enough information is
        supplied for a programmer of average skill to understand
        and use CIEX. What follows in this section is a brief
        introduction to CIEX commands to give a flavor of how
        CIEX programs may be developed.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.2.1

set $text:title = VARIABLE IDENTIFIERS
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX variable names begin with a dollar sign ($) regardless of
        where the variable appears in the program. Variable names are
        case sensitive. For example, the variable "$MYVAR" can (and
        probably will) contain a different value from the variable
        "$Myvar" or "$myvar". Subscripts in CIEX are denoted with a
        colon (:) character, such as $a:1, $a:2, $a:3 (etc.). Subscripts
        may be numbers, names, or a combination of both. Valid variable
        names include: "$table:1:1", "$table:1:2", "$employee:name", and
        "$a:b:c:d:e:f:g". Variables may be used for any single token
        anywhere in the CIEX interpreter, including replacements for
        CIEX commands and command options.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.2.2

set $text:title = CREATING VARIABLES WITH I/O REDIRECTION
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX variables can be created a number of different ways. The
        programmer may set variable values individually using the "set"
        command, may create an array from a file using the "set -file"
        option, or may create an array from the output of a command
        using the "set -cmd" option. When arrays are set, the variable
        is automatically subscripted from 1 to the number of lines in
        the command output or file. As an alternative, the "setblock"
        command allows the first token in the file or command output
        to represent the name of the subscript, thus allowing record
        structures to be automatically created. The nice thing about
        this technique is that, since the output of commands is easily
        inspected by executing the command without redirecting its
        output, the programmer can readily see the connection between
        the command and resulting data structure.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.2.3

set $text:title = CIEX FLOW OF CONTROL
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX features a full spectrum of control flow directives.
        It allows nesting of "if", "else if", "else" and "endif"
        structures to perform case selects. Operands include lexical or
        numeric comparisons, tests of object properties (such as tests
        for file existence), and compound logical operands. CIEX
        features a "loop" construct with "for" and "while" modifiers
        which may appear at the start, middle, or end of the loop. The
        "for" modifier can be a simple "for / to" expression, a
        "for -cmd" or "for -file" expression to parse the output of a
        command or contents of a file. Finally, CIEX includes a "break"
        command to break out of a loop prematurely, a "break -all"
        option to break out of all nested loops, and a "break -next"
        to immediately return to the top of a loop.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.2.4

set $text:title = MATHEMATICAL EXPRESSION EVALUATION
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX relies on a simple mechanism to distinguish between
        ordinary text strings and mathematical expressions. Many
        interpreters impose confusing syntactical rules to distinguish
        between string constants and mathematical equations. By default,
        CIEX assumes all strings are text strings, and performs
        assignments to variables accordingly. To tell the interpreter
        to treat the string as a mathematical expression and calculate
        the result, CIEX employs "set -eval", "if -eval", "while -eval"
        and "echo -eval" options. These options cause the interpreter
        to treat the operand as a math statement and return the result.
        Also, inline evaluation of a math statement can be triggered
        anywhere in a program by surrounding the string expression
        with "[" and "]" bracket characters.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3

set $text:title = ADVANCED CIEX SYNTAX
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX interpreter provides a number of advanced options
        which, although simple to use, may not be of interest to the
        elementary CIEX programmer. The general philosophy of CIEX is
        to provide the most useful functions of the interpreter "up
        front" with more esoteric functions accessible by optional
        command arguments. This section provides a flavor of the more
        advanced CIEX features.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.1

set $text:title = NESTED OUTPUT REDIRECTION
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX allows nested output redirection. This means that the
        output of a subroutine can be redirected to a file, variable,
        or pipe even though the subroutine itself redirects part of the
        output. This is a powerful filtering feature, since it means
        that a calling routine need not concern itself with the
        (possibly) redirected output of the calls it makes. Likewise,
        nested source files may redirect data to variables, files, or
        pipes without conflict to the sourcing CIEX program.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.2

set $text:title = BLOCK STATEMENTS
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        CIEX can be considered a block oriented programming language.
        One of the most powerful aspects of CIEX is that a portion of
        the CIEX program (or the entire program for that matter) can be
        blocked-off, redirected to a variable or file, or made immune to
        changes in the program environment. The "startblock" and
        "endblock" directives allow the enclosed block to be redirected
        to a variable, file, or pipe as described above. Blocks can be
        nested, thereby creating a hierarchy of redirection with lower
        level routines filtering the data used by higher level routines.
        Additionally, CIEX employs an "echoblock" command to output
        blocks of text to the screen, to a file, or to a pipe. Finally,
        CIEX includes a "setblock" command which allows entire blocks
        of variables to be set in one succinct operation.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.3

set $text:title = CALLPOINT STATEMENTS
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The basic mechanism for implementing calls to subroutines is the
        "callpoint" statement. Callpoints indicate specific locations
        in the code at which the program may start. The callpoint may
        be accessed with the "call" or "call -goto" statement. The
        former command causes the routine to stop execution at the
        current location, resume execution at the specified callpoint
        until a "return" or another call point is detected, and then
        return to the command immediately following the "call". The
        latter command causes the program to continue the program at
        the callpoint and not return to the original location. CIEX
        uses the "source -calls" command to make callpoints in an
        external file accessible to a main CIEX program.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.4

set $text:title = USING DATA ATTRIBUTES
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        Because CIEX is not a heavily typed language, it may seem that
        data types are not relevant. But it is important to distinguish
        between the fact that variables may take on any value, and that
        a particular type of value currently exists in a variable. To
        distinguish between these concepts, CIEX refers to the current
        characteristics of a value as a variable "attribute". Attributes
        of variables (such as "type", "size", "name", "next", "previous")
        are immediately available for any variable. Because variable
        attributes are free to change, the ability to determine the
        current attribute of a variable is a powerful mechanism for the
        programmer, and is a unique way of looking at the programming
        task.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.5

set $text:title = USING DATA VIEWS
set $text:page = 5
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        Multidimensional arrays and data structures are often referred
        to as "views" in the CIEX nomenclature. This is because CIEX
        allows the variable to be entered into much as one might
        enter into a disk directory. While not immediately obvious,
        multidimensional arrays and structures can easily be equated to
        directories of variables. Changing the variable directory limits
        the view of the variable just like a disk directory limits the
        visibility of the disk files (hence the term "view".) Views are
        set using the "set -view" command. To return to the top level
        view, the "unset -view" command may be used.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.6

set $text:title = SAVING DATA TO THE DISK
set $text:page = 6
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        As stated previously, the "setblock" command will allow a
        variable structure to be set from a file. The CIEX interpreter
        allows the dumping of the entire CIEX data tree (or a portion
        thereof) to a disk file in ASCII format, to be analyzed,
        processed, and/or read back into the same or different CIEX
        program. This affords a simple and powerful way to save data,
        arrays of structures (etc.) and provides a mechanism to allow
        CIEX to act as a database manager.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.3.3.7

set $text:title = THE DOS COMMAND INTERFACE
set $text:page = 7
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        No discussion of the syntax of CIEX can be complete without
        discussing the DOS command interface. CIEX builds on the Unix
        philosophy, which relies heavily on external utilities to parse
        and analyze data. This greatly extends the range available to
        the programmer, since the output of any external command can
        be fed back into the interpreter in the form of an array or
        record. (Consider the capabilities of the DOS "find" or "sort"
        routine when used in conjunction with CIEX's ability to pipe to
        these commands, and feed the result immediately into a named
        variable.) As important, CIEX can pipe keyboard input to external
        applications, thereby driving an external programs just as they
        would normally be by an operator. This inline redirection capability
        is especially useful for software test and process control.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4

set $text:title = THE BEST THING ABOUT CIEX ...
set $text:page = 0

set -cmd $text:menu  = echoblock   # # # Start menu
The Very Nicest Thing About CIEX Is...
Other Product Information
Previous Topic List
endblock                           # # # End menu

set -cmd $text:content = echoblock -wrap $margin

        If you've reviewed the material in this program, you will
        probably agree that CIEX is an extremely powerful tool. More
        than that, CIEX is at the forefront of a new technology; a
        different way of thinking about programming. CIEX can greatly
        decrease programming time, and cost. HOWEVER, here is still
        one VERY important and nice aspect of CIEX still to be
        discussed...

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.1

set $text:title = THE VERY NICEST THING ABOUT CIEX IS...
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin


endblock
set $text:content = 0

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.1.1

set $text:title =  CIEX IS ABSOLUTELY FREE!
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        ... In fact, as part of this distribution you have a fully
        functional CIEX interpreter. You are using it now! With this
        copy of the CIEX interpreter, you can create your own scripts,
        and begin to explore a new and exciting world of Very High
        Level Language programming. We sincerely hope you will use CIEX
        to reduce your software development costs, improve software
        reliability, and help advance the state-of-art for software
        design.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.1.2

set $text:title =  YOUR LICENCE TO DRIVE CIEX
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX program is protected under the copyright laws of the
        United States of America. You may make unlimited copies of the
        CIEX interpreter and demonstration files for distribution with
        CIEX scripts which you write. You can charge any fee for the
        programs you write for CIEX. However, CIEX itself cannot be
        resold for profit by any organization or individual without
        permission of New Line Software, Inc. Also, we respectfully
        REQUIRE that any CIEX programs you write for sale or for general
        distribution identify CIEX as a product of New Line Software, Inc.
        Copyright 1993, 1994.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.1.3

set $text:title =  AN IMPORTANT NOTE ABOUT YOUR COPY OF CIEX
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        New Line Software guarantees that the CIEX distribution files
        (as received directly from our offices) are free of any
        malicious software or intentionally inserted viruses. Otherwise,
        New Line Software provides no warranty of any kind, express
        or implied, with respect to the operation of CIEX, including
        without limitation any warranties of merchantability and/or
        fitness of this demonstration software for a particular purpose.
        NLS shall not be liable for any damages, whether direct,
        indirect, special or consequential arising from a failure of
        the software to operate in the manner desired by the user. NLS
        shall not be liable for any damage to data or property which
        may be caused directly or indirectly by use of the program.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.2

set $text:title = OTHER PRODUCT INFORMATION
set $text:page = 0
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin


endblock
set $text:content = 0

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.2.1

set $text:title = THE CIEX COMMAND REFERENCE MANUAL  (~$40.00)
set $text:page = 1
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX scripts in this distribution are well commented. Used
        in conjunction with the README file, included with the
        distribution files, a programmer of average skill will be able
        to write simple or moderately complex CIEX scripts. But, to
        unlock the FULL POWER of the CIEX interpreter, you will want
        the formal explanation of CIEX provided in the 250+ page
        Command Reference Manual. This manual contains detailed
        narratives and tutorial information, a complete explanation of
        grammar, a description of all CIEX commands, options, command
        usage and potential user errors, and many examples. The CIEX
        Command Reference manual is a MUST for programmers serious about
        taking full advantage of CIEX.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.2.2

set $text:title = THE CIEX DEVELOPMENT ENVIRONMENT (~$35.00)
set $text:page = 2
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The CIEX interpreter is simple to use. You can be productive
        using only the single "CIEX.EXE" executable included in this
        distribution. However, if you are interested in developing
        sophisticated, complex systems using CIEX, or crafting
        high-powered CIEX scripts as fast and as easily as possible, you
        will want the CIEX Development Environment. This suite of
        programs and utilities includes a full-screen debugger which
        allows programmers to single-step programs, set
        break points, and examine/modify data as CIEX programs run. The
        CIEX Development Environment also includes syntax checkers, an
        on-line help facility of all CIEX commands and options, script
        libraries of common utilities, and CIEX program examples.
        The CIEX Development Library is 2.5 Mbytes of text, executable
        files, and supporting documentation.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.2.3

set $text:title = THE CIEX LANGUAGE DEVELOPMENT CALL LIBRARY
set $text:page = 3
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        The most unique and powerful aspect of CIEX is its
        extensibility. A complete discussion of this special feature
        is, unfortunately, beyond the scope of this demo program.
        However, programmers should note that CIEX may be used as a
        generic command line interface for virtually any program or
        object-library, with application in process control, hardware
        and software test, and a myriad of other situations. Using
        CIEX extensibility yields what is truly a different way of
        thinking about program development. New Line Software is happy
        to discuss these features, and send detailed information at no
        charge to individuals and organizations interested in language
        development, software reliability, process automation, and
        risk reduction.

endblock

# # # # # # # # # # # # # # # # # # # # # # # # # # # #

callpoint: A.4.2.4

set $text:title = NEW LINE SOFTWARE CONSULTING SERVICES
set $text:page = 4
set $text:menu = 0

set -cmd $text:content = echoblock -wrap $margin

        New Line Software is ready to assist you with your
        project or applications. We are experts in developing CIEX
        programs and system software. We want to make you an expert
        also! New Line Software Consulting Services give you a head
        start in the development process by providing certified
        engineering and consultation services. If you have special
        questions about the CIEX interpreter, or need special
        CIEX applications developed, call us! We will save you
        time and cut your costs!

endblock

# # # # # # # # # # # # # # # # # # # # # # # # #
callpoint: table_of_contents

echoblock

:1	<view>
:1:level	0
:1:menu_default	1
:1:menu_range	5
:1:sec_name	A
:1:text_range	6
:1:title	AN INTRODUCTION TO THE CIEX INTERPRETER
:2	<view>
:2:level	1
:2:menu_default	1
:2:menu_range	4
:2:sec_name	A.1
:2:text_range	6
:2:title	WHAT IS CIEX?
:3	<view>
:3:level	2
:3:menu_default	1
:3:menu_range	0
:3:sec_name	A.1.1
:3:text_range	12
:3:title	A COMMAND INTERPRETER
:4	<view>
:4:level	3
:4:menu_default	1
:4:menu_range	0
:4:sec_name	A.1.1.1
:4:text_range	10
:4:title	INTRODUCTION TO CIEX SYNTAX
:5	<view>
:5:level	3
:5:menu_default	1
:5:menu_range	0
:5:sec_name	A.1.1.2
:5:text_range	9
:5:title	STRUCTURED PROGRAMMING
:6	<view>
:6:level	3
:6:menu_default	1
:6:menu_range	0
:6:sec_name	A.1.1.3
:6:text_range	12
:6:title	ADVANCED FEATURES
:7	<view>
:7:level	3
:7:menu_default	1
:7:menu_range	0
:7:sec_name	A.1.1.4
:7:text_range	12
:7:title	EXTENSIBLE INTERFACE
:8	<view>
:8:level	2
:8:menu_default	1
:8:menu_range	0
:8:sec_name	A.1.2
:8:text_range	9
:8:title	A DIFFERENT WAY OF PROGRAMMING
:9	<view>
:9:level	3
:9:menu_default	1
:9:menu_range	0
:9:sec_name	A.1.2.1
:9:text_range	10
:9:title	BRIEF DATA TYPING
:10	<view>
:10:level	3
:10:menu_default	1
:10:menu_range	0
:10:sec_name	A.1.2.2
:10:text_range	10
:10:title	DATA ABSTRACTION
:11	<view>
:11:level	3
:11:menu_default	1
:11:menu_range	0
:11:sec_name	A.1.2.3
:11:text_range	9
:11:title	DATA REDIRECTION
:12	<view>
:12:level	3
:12:menu_default	1
:12:menu_range	0
:12:sec_name	A.1.2.4
:12:text_range	13
:12:title	HIGH LEVEL FUNCTIONS
:13	<view>
:13:level	2
:13:menu_default	1
:13:menu_range	0
:13:sec_name	A.1.3
:13:text_range	12
:13:title	A DIFFERENT WAY OF THINKING
:14	<view>
:14:level	3
:14:menu_default	1
:14:menu_range	0
:14:sec_name	A.1.3.1
:14:text_range	12
:14:title	THE CIEX PHILOSOPHY OF INTERPRETERS
:15	<view>
:15:level	3
:15:menu_default	1
:15:menu_range	0
:15:sec_name	A.1.3.2
:15:text_range	8
:15:title	CIEX IN THE ABSTRACT
:16	<view>
:16:level	1
:16:menu_default	1
:16:menu_range	4
:16:sec_name	A.2
:16:text_range	6
:16:title	WHY CIEX?
:17	<view>
:17:level	2
:17:menu_default	1
:17:menu_range	0
:17:sec_name	A.2.1
:17:text_range	9
:17:title	CIEX FOR SMALL SCALE DEVELOPMENT
:18	<view>
:18:level	3
:18:menu_default	1
:18:menu_range	0
:18:sec_name	A.2.1.1
:18:text_range	8
:18:title	CIEX AS A SCRIPTING LANGUAGE
:19	<view>
:19:level	3
:19:menu_default	1
:19:menu_range	0
:19:sec_name	A.2.1.2
:19:text_range	9
:19:title	SIMILARITIES TO DOS BATCH FILES
:20	<view>
:20:level	3
:20:menu_default	1
:20:menu_range	0
:20:sec_name	A.2.1.3
:20:text_range	12
:20:title	SIMILARITIES TO UNIX SHELL SCRIPTS
:21	<view>
:21:level	3
:21:menu_default	1
:21:menu_range	0
:21:sec_name	A.2.1.4
:21:text_range	12
:21:title	DIFFERENCES FROM OTHER LANGUAGES
:22	<view>
:22:level	2
:22:menu_default	1
:22:menu_range	0
:22:sec_name	A.2.2
:22:text_range	6
:22:title	CIEX FOR MEDIUM SCALE DEVELOPMENT
:23	<view>
:23:level	3
:23:menu_default	1
:23:menu_range	0
:23:sec_name	A.2.2.1
:23:text_range	12
:23:title	DATABASE APPLICATION PROGRAMMING
:24	<view>
:24:level	3
:24:menu_default	1
:24:menu_range	0
:24:sec_name	A.2.2.2
:24:text_range	11
:24:title	DATA CONVERSION PROGRAMMING
:25	<view>
:25:level	3
:25:menu_default	1
:25:menu_range	0
:25:sec_name	A.2.2.3
:25:text_range	12
:25:title	MATHEMATICAL AND SCIENTIFIC PROGRAMMING
:26	<view>
:26:level	3
:26:menu_default	1
:26:menu_range	0
:26:sec_name	A.2.2.4
:26:text_range	12
:26:title	OFFICE AUTOMATION
:27	<view>
:27:level	2
:27:menu_default	1
:27:menu_range	0
:27:sec_name	A.2.3
:27:text_range	7
:27:title	CIEX FOR LARGE SCALE DEVELOPMENT
:28	<view>
:28:level	3
:28:menu_default	1
:28:menu_range	0
:28:sec_name	A.2.3.1
:28:text_range	13
:28:title	SYSTEM PROGRAMMING
:29	<view>
:29:level	3
:29:menu_default	1
:29:menu_range	0
:29:sec_name	A.2.3.2
:29:text_range	10
:29:title	SYSTEM CONFIGURATION MANAGEMENT
:30	<view>
:30:level	3
:30:menu_default	1
:30:menu_range	0
:30:sec_name	A.2.3.3
:30:text_range	11
:30:title	LARGE SCALE AUTOMATION
:31	<view>
:31:level	3
:31:menu_default	1
:31:menu_range	0
:31:sec_name	A.2.3.4
:31:text_range	11
:31:title	EXTENSIBILITY
:32	<view>
:32:level	1
:32:menu_default	1
:32:menu_range	4
:32:sec_name	A.3
:32:text_range	5
:32:title	HOW IS CIEX USED?
:33	<view>
:33:level	2
:33:menu_default	1
:33:menu_range	0
:33:sec_name	A.3.1
:33:text_range	8
:33:title	GETTING STARTED WITH CIEX
:34	<view>
:34:level	3
:34:menu_default	1
:34:menu_range	0
:34:sec_name	A.3.1.1
:34:text_range	12
:34:title	CREATING A CIEX PROGRAM
:35	<view>
:35:level	3
:35:menu_default	1
:35:menu_range	0
:35:sec_name	A.3.1.2
:35:text_range	9
:35:title	EXECUTING A CIEX PROGRAM
:36	<view>
:36:level	2
:36:menu_default	1
:36:menu_range	0
:36:sec_name	A.3.2
:36:text_range	4
:36:title	BASIC CIEX SYNTAX
:37	<view>
:37:level	3
:37:menu_default	1
:37:menu_range	0
:37:sec_name	A.3.2.1
:37:text_range	10
:37:title	VARIABLE IDENTIFIERS
:38	<view>
:38:level	3
:38:menu_default	1
:38:menu_range	0
:38:sec_name	A.3.2.2
:38:text_range	13
:38:title	CREATING VARIABLES WITH I/O REDIRECTION
:39	<view>
:39:level	3
:39:menu_default	1
:39:menu_range	0
:39:sec_name	A.3.2.3
:39:text_range	12
:39:title	CIEX FLOW OF CONTROL
:40	<view>
:40:level	3
:40:menu_default	1
:40:menu_range	0
:40:sec_name	A.3.2.4
:40:text_range	12
:40:title	MATHEMATICAL EXPRESSION EVALUATION
:41	<view>
:41:level	2
:41:menu_default	1
:41:menu_range	0
:41:sec_name	A.3.3
:41:text_range	6
:41:title	ADVANCED CIEX SYNTAX
:42	<view>
:42:level	3
:42:menu_default	1
:42:menu_range	0
:42:sec_name	A.3.3.1
:42:text_range	8
:42:title	NESTED OUTPUT REDIRECTION
:43	<view>
:43:level	3
:43:menu_default	1
:43:menu_range	0
:43:sec_name	A.3.3.2
:43:text_range	12
:43:title	BLOCK STATEMENTS
:44	<view>
:44:level	3
:44:menu_default	1
:44:menu_range	0
:44:sec_name	A.3.3.3
:44:text_range	11
:44:title	CALLPOINT STATEMENTS
:45	<view>
:45:level	3
:45:menu_default	1
:45:menu_range	0
:45:sec_name	A.3.3.4
:45:text_range	11
:45:title	USING DATA ATTRIBUTES
:46	<view>
:46:level	3
:46:menu_default	1
:46:menu_range	0
:46:sec_name	A.3.3.5
:46:text_range	9
:46:title	USING DATA VIEWS
:47	<view>
:47:level	3
:47:menu_default	1
:47:menu_range	0
:47:sec_name	A.3.3.6
:47:text_range	7
:47:title	SAVING DATA TO THE DISK
:48	<view>
:48:level	3
:48:menu_default	1
:48:menu_range	0
:48:sec_name	A.3.3.7
:48:text_range	13
:48:title	THE DOS COMMAND INTERFACE
:49	<view>
:49:level	1
:49:menu_default	1
:49:menu_range	3
:49:sec_name	A.4
:49:text_range	6
:49:title	THE BEST THING ABOUT CIEX ...
:50	<view>
:50:level	2
:50:menu_default	1
:50:menu_range	0
:50:sec_name	A.4.1
:50:text_range	0
:50:title	THE VERY NICEST THING ABOUT CIEX IS...
:51	<view>
:51:level	3
:51:menu_default	1
:51:menu_range	0
:51:sec_name	A.4.1.1
:51:text_range	7
:51:title	CIEX IS ABSOLUTELY FREE!
:52	<view>
:52:level	3
:52:menu_default	1
:52:menu_range	0
:52:sec_name	A.4.1.2
:52:text_range	9
:52:title	YOUR LICENCE TO DRIVE CIEX
:53	<view>
:53:level	3
:53:menu_default	1
:53:menu_range	0
:53:sec_name	A.4.1.3
:53:text_range	11
:53:title	AN IMPORTANT NOTE ABOUT YOUR COPY OF CIEX
:54	<view>
:54:level	2
:54:menu_default	1
:54:menu_range	0
:54:sec_name	A.4.2
:54:text_range	0
:54:title	OTHER PRODUCT INFORMATION
:55	<view>
:55:level	3
:55:menu_default	1
:55:menu_range	0
:55:sec_name	A.4.2.1
:55:text_range	11
:55:title	THE CIEX COMMAND REFERENCE MANUAL ($40.00)
:56	<view>
:56:level	3
:56:menu_default	1
:56:menu_range	0
:56:sec_name	A.4.2.2
:56:text_range	13
:56:title	THE CIEX DEVELOPMENT ENVIRONMENT ($35.00)
:57	<view>
:57:level	3
:57:menu_default	1
:57:menu_range	0
:57:sec_name	A.4.2.3
:57:text_range	11
:57:title	THE CIEX LANGUAGE DEVELOPMENT CALL LIBRARY
:58	<view>
:58:level	3
:58:menu_default	1
:58:menu_range	0
:58:sec_name	A.4.2.4
:58:text_range	8
:58:title	NEW LINE SOFTWARE CONSULTING SERVICES
:max_secno	58
:output_file	INTRO.S
endblock

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: interrupt

# # # # # # # # # # # # # # # # # # # # # #

return -entry

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the intro.s program.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: intro.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"
move -down; echo -scr "Press any key to continue..."
read -key $arg:exit

return -entry

