# # # # # # # # # # # # # # # # # # # # # # #

# REACTOR.S - GENERAL FUSION, INC. Reactor Control Panel.
# CIEX (C) 1993, 1994 New Line Software, Inc.

# NLS, Inc. New Line Software
# P.O. Box 3433, Englewood, CO 80155

# Offices at:
# 7348 S. Alton Way, Unit I, Englewood, CO 80112

# Phone: 1-800-441-2931, Fax: 1-303-771-0836

# In Denver, and outside of USA,
# Phone: 1-303-741-6594

# Display a control panel for a Fusion Reactor.

# # # # # # # # # # # # # # # # # # # # # # #

# Initialize the program environment.

set -clear        # Clear all variables and callpoints from the program.
screen -def       # Initial screen parameters are default.
source -calls     # Speed up access to callpoints in this file.

# The list of screens callpoints are identified below, where each
# top level screen is identified by a consecutive integer screen number.
# This list may be augmented by the user to add other screens to the 
# program.  See the "run_template" callpoint for details.

setblock $screen_list

1  run_profile_screen
2  run_schematic_screen
3  run_transients_screen
4  run_strip_chart
5  run_flux_screen

endblock

# The first screen displayed is as follows:

set $screen_number = $screen_list^first

# # # # # # # # # #

callpoint: run_profile_screen

# Run the reactor profile screen.

# # # # # # # # # #

unset -view
set -view $profile
unset -all
trace -key

select -curs off
screen -buff off
screen -clear

call draw_profile
call read_sensors

loop

  call read_sensors
  call update_profile
  sleep 1000

endloop
return

# # # # # # # # # #

callpoint: run_schematic_screen

# Run the reactor schematic screen.

# # # # # # # # # #

unset -view
set -view $schematic
unset -all
trace -key
select -curs off
screen -buff off

call draw_schematic
loop
   loop for $i = 1 to 30
      call route_sensors $i
      call update_schematic $i
      sleep 1000
   endloop
endloop
read -key $a
return

# # # # # # # # # #

callpoint: run_transients_screen

# Run the reactor transient analysis screen.

# # # # # # # # # #

unset -view
set -view $transients
unset -all
trace -key

select -curs off
screen -buff off
screen -clear

set $orig = 4

call draw_transients
call update_trans_table

loop
   call update_trans_screen
   call update_trans_table
   sleep 1000
endloop
return

# # # # # # # # # #

callpoint: run_strip_chart

# Run the reactor transient analysis screen.

# # # # # # # # # #

unset -view
set -view $strip_chart
unset -all
trace -key

select -curs off
screen -buff off
screen -clear

set -inlin $vbar = echo -ascii 179
call draw_strip_chart
loop
   call update_strip_chart
   sleep 1000
endloop

return

# # # # # # # # # #

callpoint: run_flux_screen

# Run the reactor helio flux screen.

# # # # # # # # # #

unset -view
set -view $flux
unset -all
trace -key

screen -def

select -curs off
screen -buff off
screen -clear

set $flux_value = 0
call draw_flux_screen
loop
   call update_flux_screen
   sleep 1500
endloop

# # # # # # # # # #

callpoint: draw_profile

# Draw the reactor profile screen.

# # # # # # # # # #

# Draw the control bar at the top of the display:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

# Main headers:

scr -fore cyan
move 12 3      ; echo -scr 'REACTOR CORE # 1'
move -right 40 ; echo -scr 'REACTOR CORE # 2'

# Draw the bar graphs:

screen -foreground red
box -style single

box  7 5 3 13
box 17 5 3 13
box 27 5 3 13
box 37 5 3 13
box 47 5 3 13
box 57 5 3 13
box 67 5 3 13

# Label the bar graphs:

screen -foreground green
move  6  4     ; echo -scr '8.2 KP'
move -right 10 ; echo -scr '8.2 KN'
move -right 10 ; echo -scr '8.2 MC'
move -right 10 ; echo -scr '8.2 DL'
move -right 10 ; echo -scr '8.2 KP'
move -right 10 ; echo -scr '8.2 KN'
move -right 10 ; echo -scr '8.2 MC'

move  6 18     ; echo -scr '0.1 KP'
move -right 10 ; echo -scr '0.1 KN'
move -right 10 ; echo -scr '0.1 MC'
move -right 10 ; echo -scr '0.1 DL'
move -right 10 ; echo -scr '0.1 KP'
move -right 10 ; echo -scr '0.1 KN'
move -right 10 ; echo -scr '0.1 MC'

# Draw the table:

screen -foreground red

move 12 19
move -down 1   ; echo -scr KP/S
move -right 38 ; echo -scr KP/S
move -left  38

move -down 1   ; echo -scr KN/S
move -right 38 ; echo -scr KN/S
move -left  38

move -down 1   ; echo -scr MK C
move -right 38 ; echo -scr MK C
move -left  38

# # # # # # # # # # # # # # #

callpoint: draw_schematic

# Draw the reactor schematic screen.

# # # # # # # # # # # # # # #

screen -def
screen -clear

# Initialize the following special symbols.

set -inline $right_arrow = echo -ascii 16
set -inline $left_arrow  = echo -ascii 17
set -inline $sum         = echo -ascii 228
set -inline $delta       = echo -ascii 235

set -inline $up_arrow    = echo -ascii 24
set -inline $down_arrow  = echo -ascii 25

# Draw the control bar at the top of the display:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

# Reactor 1 Plants:

screen -foreground red
box -style single

box 10  3 11  4
box 30  3 11  4
box 50  3 11  4

# Reactor 2 Plants:

box 10 20 11  4
box 30 20 11  4
box 50 20 11  4

# Reactor core:

box 68 12  5  3
box 72  8  5 11

# Horizontal connectivity

screen -fore white

move  2  4; line -horizontal  8
move  2 21; line -horizontal  8
move 21  4; line -horizontal  9
move 21 21; line -horizontal  9
move 41  4; line -horizontal  9
move 41 21; line -horizontal  9
move 61  4; line -horizontal  6
move 61 21; line -horizontal  6

move  6 12; line -horizontal 40
move  6 14; line -horizontal 40
move 25 10; line -horizontal 21
move 25 16; line -horizontal 21
move 66 10; line -horizontal  6
move 66 16; line -horizontal  6
move 52 13; line -horizontal 16

# Vertical connectivity

move  6  4; line  -vertical  9
move  6 14; line  -vertical  8
move 25  4; line  -vertical  7
move 25 16; line  -vertical  6
move 51 10; line  -vertical  7
move 66  4; line  -vertical  7
move 66 16; line  -vertical  6

# Arrows:

move  9  4; echo -scr $right_arrow
move 29  4; echo -scr $right_arrow
move 49  4; echo -scr $right_arrow
move 71 10; echo -scr $right_arrow
move 71 16; echo -scr $right_arrow
move  9 21; echo -scr $right_arrow
move 29 21; echo -scr $right_arrow
move 49 21; echo -scr $right_arrow
move 52 13; echo -scr $left_arrow

# Summations

screen -fore cyan

move  6  4 ; echo -scr $sum
move 25  4 ; echo -scr $sum
move  6 21 ; echo -scr $sum
move 25 21 ; echo -scr $sum

# Labels:

move 28 7    ; echo -scr 'REACTOR CORE # 1'
move -down 12; echo -scr 'REACTOR CORE # 2'

move 14 4    ; echo -scr 'VSM'
move -down 17; echo -scr 'VSM'

move 32 4    ; echo -scr 'SEC TVM'
move -down 17; echo -scr 'SEC TVM'

move 52 4    ; echo -scr 'QX COND'
move -down 17; echo -scr 'QX COND'

move 40  9; echo -scr "A"
move 40 11; echo -scr "B"
move 40 13; echo -scr "C"
move 40 15; echo -scr "D"

move 74 10; echo -scr "P"
move 74 16; echo -scr "N"

# # # # # # # # # # # # # #

callpoint: draw_transients

# Draw the reactor transients screen.

# # # # # # # # # # # # # #

set $j = 0

# Initialize the screen:

screen -def
select -curs off

# Draw the control bar at the top of the display:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

# Create the view box:

screen -fore red
box -style single
box  12 [$orig - 1] 59 10
box -style clear

move 11 [ $orig + 4 ]; line -horiz 2
move 70 [ $orig + 4 ]; line -horiz 2
move 40 [ $orig - 2 ]; line -vert 2
move 40 [ $orig + 8 ]; line -vert 2

move 40 [$orig + 9] ; echo -scr " "
move 40 [$orig - 2] ; echo -scr " "

# Draw the table:

move 10 15;   line 63
move -down 2; line 63
loop for $x =  29 to 60 by 22
   move $x 15
   line -vertical 8
endloop

loop for $x =  10 to 60 by 22
   loop for $y = 18 to 22 by 2
      set $j + 1
      move $x $y; echo -scr "A $j :"
   endloop
endloop

# Add labels to the display:

scr -fore cyan
move 13 16     ; echo -scr "PVX PROF A"
move -right 22 ; echo -scr "QVX PROF A"
move -right 22 ; echo -scr "MVX PROF A"

screen -fore green
move 30  14    ; echo -scr "AVX TRANSIENT PROFILE"

# # # # # # # # # # # # # #

callpoint: draw_strip_chart

# Draw the strip chart recorder screen.

# # # # # # # # # # # # # #

# Draw the control bar at the top of the display:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

screen -fore red
box -style single

# Reactor 1

box  3  6 14 7
box 18  6 14 7
box 33  6 14 7
box 48  6 14 7
box 63  6 14 7

# Reactor 2

box  3 16 14 7
box 18 16 14 7
box 33 16 14 7
box 48 16 14 7
box 63 16 14 7

screen -fore cyan

scr -fore cyan
move  8  4; echo -scr 'REACTOR CORE # 1'

scr -fore green
move  3  5; echo -scr12 "CRS/SEC"
move 18  5; echo -scr12 "VRS/SEC"
move 33  5; echo -scr12 "XMS/SEC"
move 48  5; echo -scr12 "X2S/SEC"
move 63  5; echo -scr12 "X3S/SEC"

scr -fore cyan
move  8 14; echo -scr 'REACTOR CORE # 2'

scr -fore green
move  3 15; echo -scr12 "CRS/SEC"
move 18 15; echo -scr12 "VRS/SEC"
move 33 15; echo -scr12 "XMS/SEC"
move 48 15; echo -scr12 "X2S/SEC"
move 63 15; echo -scr12 "X3S/SEC"

screen -fore white
return
loop for $i = 4 to 78 by 15
   loop for $j = 7  to 11
      move $i $j
      echo -scr12 "[fix4(sin($i+$j) * 100))] "
   endloop
endloop

loop for $i = 4 to 78 by 15
   loop for $j = 17 to 21
      move $i $j
      echo -scr12 "[fix4(sin($i+$j) * 100))] "
   endloop
endloop

# # # # # # # # # #

callpoint: draw_flux_screen

# Draw the helio flux screen.

# # # # # # # # # #

scr -def
scr -back black
scr -clear

set -inlin $arrow_character = echo -ascii 31

# Top legend:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

# Graph box:

scr -fore cyan
move 1 3; echo -center "HELIO FLUX DISTRIBUTION (PCT)"

scr -fore red
box -style single
box 15 4 51 15
scr -fore red
scr -back black

# Right scale

scr -fore cyan
move 67 3
move -down 2; echo -scr "SW A"
move -down 2; echo -scr "SW B"
move -down 2; echo -scr "SW C"
move -down 2; echo -scr "SW D"
move -down 2; echo -scr "FW Z"
move -down 2; echo -scr "RE S"
move -down 2; echo -scr "AX Z"

# Bottom scale

scr -fore white
move 15 21; line 51

move 10 21
loop for $scale_value = 0 to 100 by 10

   move -right 5; line -vert 2
   move -down   ;echo -scr $scale_value
   move -up

endloop

return

# # # # # # # # # #

callpoint: update_profile

# Update the profile display with values

# # # # # # # # # #

screen -foreground white
move 22 19

# Update the table:

move -down 1   ; echo -scr $kps1
move -right 38 ; echo -scr $kps2
move -left  38

move -down 1   ; echo -scr $kns1
move -right 38 ; echo -scr $kns2
move -left  38

move -down 1   ; echo -scr $mkc1
move -right 38 ; echo -scr $mkc1
move -left  38

# Update the bar graphs.

scr -fore red
move 8 8   ; move -down [int $kps1]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 10 8  ; move -down [int $kps1]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $kps1) ]
move -down ; line -clear 6

scr -fore red
move 18 8  ; move -down [int $kns1]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 20 8  ; move -down [int $kns1]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $kns1) ]
move -down ; line -clear 6

scr -fore red
move 28 8  ; move -down [int $mkc1]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 30 8  ; move -down [int $mkc1]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $mkc1) ]
move -down ; line -clear 6

scr -fore red
move 38 8  ; move -down [int $delt]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 40 8  ; move -down [int $delt]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $delt) ]
move -down ; line -clear 6

scr -fore red
move 48 8  ; move -down [int $kps2]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 50 8  ; move -down [int $kps2]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $kps2) ]
move -down ; line -clear 6

scr -fore red
move 58 8  ; move -down [int $kns2]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 60 8  ; move -down [int $kns2]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $kns2) ]
move -down ; line -clear 6

scr -fore red
move 68 8  ; move -down [int $mkc2]
move -up   ; line -clear 1;
move -down ; line -clear 1; line -invert 1
move -down ; line -clear 1;

scr -fore white
move 70 8  ; move -down [int $mkc2]
move -up   ; line -clear 6
move -down ; echo -scr [ fix3 (8 - $mkc2) ]
move -down ; line -clear 6

return

# # # # # # # # # # # # # # #

callpoint: update_schematic

# Update the schematic display with values.

# # # # # # # # # # # # # # #

screen -fore green

move  4  2 ; echo -scr4 "[ fix2 (cos ($arg:1 + 1)) ] "
move -down ; echo -scr4 KMS
move 24  2 ; echo -scr4 "[ fix2 (cos ($arg:1 + 2)) ] "
move -down ; echo -scr4 PKK
move  4 22 ; echo -scr4 "[ fix2 (cos ($arg:1 + 3)) ] "
move -down ; echo -scr4 KMS
move 24 22 ; echo -scr4 "[ fix2 (cos ($arg:1 + 4)) ] "
move -down ; echo -scr4 PKK

move 68  7 ; echo -scr5 [ log ($arg:1 + 11) ]
move -right 7; echo -scr "TS"

move 68 19 ; echo -scr5 [log ($arg:1 + 8) ]
move -right 7; echo -scr "TS"

move 54 12 ; echo -scr5 "[ abs(fix4 (sin ($arg:1) * 122)) ]  "
move -right 7 ; echo -scr "$delta KWT"

# # # # # # # # # # # # # #

callpoint: update_trans_table

# Update the transient analysis table with values.

# # # # # # # # # # # # # #

screen -fore white

loop for $x = 15 to 65 by 22
  loop for $y = 18 to 22 by 2
    set $m + 1
    set $m % 5
    move $x $y; echo -scr12 [fix3 log($m + 5)]
  endloop
endloop

# # # # # # # # # # # # # #

callpoint: update_trans_screen

# Sweep transient analysis viewport.

# # # # # # # # # # # # # #

screen -fore cyan
loop for $i = 14 to 68

   box $i $orig 2 8
   move [$i+1] $orig;  line -vertical 8
   set $k = [int ( 3 * sin($j/4) * cos($j/6) + $orig + 4)]
   move $i $k
   line -vert 1

   set $j + 1

endloop
box $i $orig 2 8
screen -fore red; move [$i+1] $orig;  line -vertical 8

# # # # # # # # # # # # # #

callpoint: update_strip_chart

# Update the strip chart recorder with values.

# # # # # # # # # # # # # #

set $mark ? 0
set $j % 50
select -screen 1 7 80 4
scroll -down
select -screen 1 17 80 4
scroll -down

screen -fore red;

move  3      7; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;

move  3     17; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;
move -right  2; echo -scr $vbar;
move -right 13; echo -scr $vbar;

screen -fore white;
loop for $i = 4 to 78 by 15
   move $i 7
   echo -scr12 "[fix4(sin($i+$j) * 100))] "
endloop

set $j + 1

loop for $i = 4 to 78 by 15
   move $i 17
   echo -scr12 "[fix4(sin($i+$j) * 100))] "
endloop
set $j + 1

# # # # # # # # # # # # # #

callpoint: update_flux_screen

# Update the helio flux screen with values.

# # # # # # # # # # # # # #

# Top line of chart

screen -fore red
box -clear 16 5 49 13

# Loop for each bar element.

move 16 5
set $total = 0

loop

   # Calculate magnitude of this sensor.

   set -eval $flux_value  = [  $flux_value + 0.3]
   set -eval $magnitude   = fix3(abs(45 * cos($flux_value))) + 2
   set -eval $scale_value = fix3($magnitude * 2)

   # Label the magnitude.

   scr -fore green
   move -left  10
   echo -scr8 "$scale_value"
   move -right 10

   # Display the magnitude.
   # Note - Line is inverted for monochrome displays.

   scr -fore red
   line -clear 49
   line  -invert [ int($magnitude) ]

for $i = 1 to 6

   move -down 2
   set -eval $total = $total + $magnitude

endloop

# Calculate average.

set -eval $average     = fix3($total / 7)
set -eval $xcoord      = int($average)
set -eval $scale_value = fix3($average * 2)

scr -fore green
move  [ $xcoord + 15 ]  5
line -vert 13

move 1 19  ; line -clear
move -down ; line -clear

move [ $xcoord + 8 ] 19
echo -scr "RMS: $scale_value"

move [ $xcoord + 15] 20
echo -scr $arrow_character

move -home

return

# # # # # # # # # #

callpoint: read_sensors

# Read the reactor sensors.

# # # # # # # # # #

if $i^value is null
  set $i = 0
endif

set -eval $i  = $i + 0.15

set -eval $kps1  = fix4 abs(5 * cos ($i + 0.51))
set -eval $kns1  = fix4 abs(5 * cos ($i + 1.01))
set -eval $mkc1  = fix4 abs(5 * cos ($i + 0.41))
set -eval $delt  = fix4 abs(5 * cos ($i + 2.4 ))
set -eval $kps2  = fix4 abs(5 * cos ($i + 0.42))
set -eval $kns2  = fix4 abs(5 * cos ($i + 1.02))
set -eval $mkc2  = fix4 abs(5 * cos ($i + 0.52))

# # # # # # # # # # # # # # #

callpoint: route_sensors

# # # # # # # # # # # # # # #

box -style clear
screen -fore red

set $arg:1 % 9
if $arg:1 le 1

   box  46  9  5  9
   move 46 10; line 5; echo -scr $left_arrow
   move 46 14; line 5; echo -scr $left_arrow
   move 74 13 ; echo -scr $up_arrow

else if $arg:1 eq 2;

   box  46  9  5  9
   move 46 12; line 5; echo -scr $left_arrow
   move 46 14; line 5; echo -scr $left_arrow
   move 74 13 ; echo -scr $down_arrow

else if $arg:1 eq 3;

   box  46  9  5  9
   move 46 10; line 5; echo -scr $left_arrow
   move 46 16; line 5; echo -scr $left_arrow
   move 74 13 ; echo -scr $up_arrow

else if $arg:1 eq 4;

   box  46  9  5  9
   move 46 10; line 5; echo -scr $left_arrow
   move 46 14; line 5; echo -scr $left_arrow
   move 74 13 ; echo -scr $down_arrow

else if $arg:1 eq 5;

   box  46  9  5  9
   move 46 12; line 5; echo -scr $left_arrow
   move 46 16; line 5; echo -scr $left_arrow
   move 74 13; echo -scr $up_arrow

else if $arg:1 eq 6;

   box  46  9  5  9
   move 46 12; line 5; echo -scr $left_arrow
   move 46 14; line 5; echo -scr $left_arrow
   move 74 13; echo -scr $down_arrow

else if $arg:1 eq 7;

   box  46  9  5  9
   move 46 10; line 5; echo -scr $left_arrow
   move 46 14; line 5; echo -scr $left_arrow
   move 74 13; echo -scr $up_arrow

else if $arg:1 eq 8

   box  46  9  5  9
   move 46 12; line 5; echo -scr $left_arrow
   move 46 16; line 5; echo -scr $left_arrow
   move 74 13; echo -scr $down_arrow

endif

# # # # # # # # # # # # # #

callpoint: trace

# Asynchronously process a function key entry:

# # # # # # # # # # # # # #

unset -view
read -key $key

if      $key eq f1   # Display the help screen.

   trace -off
   call help_screen
   trace -key

else if $key eq f3   # Display the previous screen

   # Display the previous screen.

   set $screen_number = $screen_list:$screen_number^prev
   if $screen_number is null
      set $screen_number = $screen_list^last
   endif

else if  $key eq f4  # Display the next screen

   set $screen_number = $screen_list:$screen_number^next
   if $screen_number is null
      set $screen_number = $screen_list^first
   endif

else if $key eq f10  # Exit the program.

   scr -buffer off
   scr -clear
   call -goto exit

else # Unknown key was entered. Continue.

   return

endif

# Go to the specified screen. 

call -goto $screen_list:$screen_number

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: help_screen

# # # # # # # # # # # # # # # # # # # # # #

screen -clear
box -style french

screen -fore cyan
move 1 1
line -clear
echo -center "    (Any key to return to main screen...)"
line -invert 80

scr -fore red
box 1 3 80 20

scr -fore cyan
move 5 5; echo -scr "ABOUT THE REACTOR PROGRAM."

scr -fore white
move -down  1

set -cmd $text =  echoblock

   The REACTOR program monitors a GENERAL FUSION, INC, home fusion
   power reactor (Circa 2095) using several different screens. (This
   is a typical application of the CIEX interpreter, that is,
   presenting text-graphical data.)

endblock

loop for $line = $text
   echo -scr $line; move -down
endloop

scr -fore cyan
echo -scr "FUNCTION KEYS:"
scr -fore white
move -down

set -cmd $text =  echoblock

   [ F1=Help        ]    - Display this help message.
   [ F3=Prev Screen ]    - Display the previous monitor screen.
   [ F4=Next Screen ]    - Display the next monitor screen.
   [ F10=Exit       ]    - Exit the program.

endblock

loop for $line = $text
   echo -scr $line; move -down
endloop

read -key $option

return

# # # # # # # # # #

callpoint: run_template

# A template for adding a screen.

# Copy and rename the "run_template", "draw_template", and 
# "update_template" callpoints below to add new screens to the program.
# The modified "run_template" callpoint should then be added to the 
# "$screen_list" array (defined at the top of the script), and will then
# be accessible using "Next Screen" and "Previous Screen" function keys.

# # # # # # # # # #

unset -view
set -view $template
unset -all
trace -key
select -curs off
screen -buff off

# Draw the screen outline.

call draw_template

# Enter the main service loop.

loop

   # Update the screen with new values.

   call update_template

   # Delay for one second.

   sleep 1000

endloop

# # # # # # # # # #

callpoint: draw_template

# A template for drawing a screen. 

# This routine is called once each time the "run_template" callpoint is
# accessed, and draws the static screen display.

# # # # # # # # # #

screen -def
screen -clear

# Draw the control bar at the top of the display:

screen -fore cyan
move 1 1
line -clear 80
echo -center ~
" F1=Help            F3=Prev Screen           F4=Next Screen           F10=Exit"
line -invert 80

# Draw bottom legend.

move -home
line -clear 80
echo -center "CIEX V2.3, NLS, Inc."
line -invert 80

# Add static elements of the screen here:

box 10 10
move 1 12; echo -center "Add screen draw commands here."

return

# # # # # # # # # #

callpoint: update_template

# A template for updating dynamic elements of the screen. 

# This callpoint is called once each second while the "run_template" 
# screen is active, and dynamically updates the screen.

# # # # # # # # # #

# Add dynamic commands to update the screen here.
# The following is an example, which displays an incrementing counter.

set $counter + 1
set -inline $counter = echo -int5.5 $counter
move 1 13; echo -center "Call number: $counter"

return

# # # # # # # # # # # # # # # # # # # # # #

callpoint: interrupt

# Jump here when a CNTRL-C or BREAK key is encountered during
# processing.

# # # # # # # # # # # # # # # # # # # # # #

call -goto exit

# # # # # # # # # # # # # #

callpoint: exit

# Leave the program, first unsetting any data used by the
# program.

# # # # # # # # # # # # # #

trace -off
unset -view
unset $profile
unset $schematic
unset $transients
unset $strip_chart
unset $flux
return

# # # # # # # # # # # # # # #

callpoint: exception

# Handle a problem with the reactor program.

# # # # # # # # # # # # # # #

scr -buffer off
scr -def
scr -clear
move 5 10

set $sys:errtxt ? "General failure during execution"
set $sys:errloc ? "( From: reactor.s )"

move -down; echo -scr "ERROR: Has script been improperly modified?"
move -down; echo -scr "$sys:errtxt $sys:errloc"
move -down; echo -scr "See 'README.TXT' for trouble shooting information."
move -down; echo -scr "Contact NLS, Inc. New Line Software for assistance."
move -down; echo -scr "Phone: 1-800-441-2931, Fax: 1-303-771-0836"
move -down; echo -scr "Press any key to continue..."
read -key $arg:exit

return -entry


